#![allow(unreachable_patterns)]
use std::{borrow::Borrow, ops::Deref};

use crate::backends::{
    keycodes::{MouseButton, VirtualKeyCode},
    window_event::ModifierState,
};

pub fn xev_to_mouse_button(
    button: &x11::xlib::XButtonEvent,
) -> Option<MouseButton> {
    match button.button {
        1 => Some(MouseButton::Left),
        2 => Some(MouseButton::Middle),
        3 => Some(MouseButton::Right),
        4 => Some(MouseButton::ScrollUp),
        5 => Some(MouseButton::ScrollDown),
        6 => Some(MouseButton::ScrollLeft),
        7 => Some(MouseButton::ScrollRight),
        8 => Some(MouseButton::Forward),
        9 => Some(MouseButton::Backward),
        _ => None,
    }
}

pub fn mouse_button_to_xbutton(button: MouseButton) -> i32 {
    match button {
        MouseButton::Left => 1,
        MouseButton::Middle => 2,
        MouseButton::Right => 3,
        MouseButton::ScrollUp => 4,
        MouseButton::ScrollDown => 5,
        MouseButton::ScrollLeft => 6,
        MouseButton::ScrollRight => 7,
        MouseButton::Forward => 8,
        MouseButton::Backward => 9,
    }
}

#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Copy, Clone)]
pub struct XKeySym(pub u32);

impl XKeySym {
    pub fn new(value: u32) -> Self {
        Self(value)
    }

    pub fn get(&self) -> u32 {
        self.0
    }
}

impl Borrow<u32> for XKeySym {
    fn borrow(&self) -> &u32 {
        &self.0
    }
}

impl AsRef<u32> for XKeySym {
    fn as_ref(&self) -> &u32 {
        &self.0
    }
}

impl Deref for XKeySym {
    type Target = u32;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

impl From<XKeySym> for VirtualKeyCode {
    fn from(value: XKeySym) -> Self {
        keysym_to_virtual_keycode(*value).unwrap()
    }
}
impl From<VirtualKeyCode> for XKeySym {
    fn from(value: VirtualKeyCode) -> Self {
        Self(virtual_keycode_to_keysym(value).unwrap())
    }
}

pub fn xlib_state_to_modifier_state(state: u32) -> ModifierState {
    todo!()
}

/// from winit
pub fn keysym_to_virtual_keycode(keysym: u32) -> Option<VirtualKeyCode> {
    Some(match keysym {
        x11::keysym::XK_BackSpace => VirtualKeyCode::Back,
        x11::keysym::XK_Tab => VirtualKeyCode::Tab,
        //x11::keysym::XK_Linefeed => VirtualKeyCode::Linefeed,
        //x11::keysym::XK_Clear => VirtualKeyCode::Clear,
        x11::keysym::XK_Return => VirtualKeyCode::Return,
        //x11::keysym::XK_Pause => VirtualKeyCode::Pause,
        //x11::keysym::XK_Scroll_Lock => VirtualKeyCode::Scroll_lock,
        //x11::keysym::XK_Sys_Req => VirtualKeyCode::Sys_req,
        x11::keysym::XK_Escape => VirtualKeyCode::Escape,
        x11::keysym::XK_Delete => VirtualKeyCode::Delete,
        x11::keysym::XK_Multi_key => VirtualKeyCode::Compose,
        //x11::keysym::XK_Kanji => VirtualKeyCode::Kanji,
        //x11::keysym::XK_Muhenkan => VirtualKeyCode::Muhenkan,
        //x11::keysym::XK_Henkan_Mode => VirtualKeyCode::Henkan_mode,
        //x11::keysym::XK_Henkan => VirtualKeyCode::Henkan,
        //x11::keysym::XK_Romaji => VirtualKeyCode::Romaji,
        //x11::keysym::XK_Hiragana => VirtualKeyCode::Hiragana,
        //x11::keysym::XK_Katakana => VirtualKeyCode::Katakana,
        //x11::keysym::XK_Hiragana_Katakana => VirtualKeyCode::Hiragana_katakana,
        //x11::keysym::XK_Zenkaku => VirtualKeyCode::Zenkaku,
        //x11::keysym::XK_Hankaku => VirtualKeyCode::Hankaku,
        //x11::keysym::XK_Zenkaku_Hankaku => VirtualKeyCode::Zenkaku_hankaku,
        //x11::keysym::XK_Touroku => VirtualKeyCode::Touroku,
        //x11::keysym::XK_Massyo => VirtualKeyCode::Massyo,
        //x11::keysym::XK_Kana_Lock => VirtualKeyCode::Kana_lock,
        //x11::keysym::XK_Kana_Shift => VirtualKeyCode::Kana_shift,
        //x11::keysym::XK_Eisu_Shift => VirtualKeyCode::Eisu_shift,
        //x11::keysym::XK_Eisu_toggle => VirtualKeyCode::Eisu_toggle,
        x11::keysym::XK_Home => VirtualKeyCode::Home,
        x11::keysym::XK_Left => VirtualKeyCode::Left,
        x11::keysym::XK_Up => VirtualKeyCode::Up,
        x11::keysym::XK_Right => VirtualKeyCode::Right,
        x11::keysym::XK_Down => VirtualKeyCode::Down,
        //x11::keysym::XK_Prior => VirtualKeyCode::Prior,
        x11::keysym::XK_Page_Up => VirtualKeyCode::PageUp,
        //x11::keysym::XK_Next => VirtualKeyCode::Next,
        x11::keysym::XK_Page_Down => VirtualKeyCode::PageDown,
        x11::keysym::XK_End => VirtualKeyCode::End,
        //x11::keysym::XK_Begin => VirtualKeyCode::Begin,
        x11::keysym::XK_Win_L => VirtualKeyCode::LWin,
        x11::keysym::XK_Win_R => VirtualKeyCode::RWin,
        //x11::keysym::XK_App => VirtualKeyCode::App,
        //x11::keysym::XK_Select => VirtualKeyCode::Select,
        x11::keysym::XK_Print => VirtualKeyCode::Print,
        //x11::keysym::XK_Execute => VirtualKeyCode::Execute,
        x11::keysym::XK_Insert => VirtualKeyCode::Insert,
        //x11::keysym::XK_Undo => VirtualKeyCode::Undo,
        //x11::keysym::XK_Redo => VirtualKeyCode::Redo,
        //x11::keysym::XK_Menu => VirtualKeyCode::Menu,
        //x11::keysym::XK_Find => VirtualKeyCode::Find,
        //x11::keysym::XK_Cancel => VirtualKeyCode::Cancel,
        //x11::keysym::XK_Help => VirtualKeyCode::Help,
        //x11::keysym::XK_Break => VirtualKeyCode::Break,
        //x11::keysym::XK_Mode_switch => VirtualKeyCode::Mode_switch,
        //x11::keysym::XK_script_switch => VirtualKeyCode::Script_switch,
        //x11::keysym::XK_Num_Lock => VirtualKeyCode::Num_lock,
        //x11::keysym::XK_KP_Space => VirtualKeyCode::Kp_space,
        //x11::keysym::XK_KP_Tab => VirtualKeyCode::Kp_tab,
        //x11::keysym::XK_KP_Enter => VirtualKeyCode::Kp_enter,
        //x11::keysym::XK_KP_F1 => VirtualKeyCode::Kp_f1,
        //x11::keysym::XK_KP_F2 => VirtualKeyCode::Kp_f2,
        //x11::keysym::XK_KP_F3 => VirtualKeyCode::Kp_f3,
        //x11::keysym::XK_KP_F4 => VirtualKeyCode::Kp_f4,
        x11::keysym::XK_KP_Home => VirtualKeyCode::Home,
        x11::keysym::XK_KP_Left => VirtualKeyCode::Left,
        x11::keysym::XK_KP_Up => VirtualKeyCode::Up,
        x11::keysym::XK_KP_Right => VirtualKeyCode::Right,
        x11::keysym::XK_KP_Down => VirtualKeyCode::Down,
        //x11::keysym::XK_KP_Prior => VirtualKeyCode::Kp_prior,
        x11::keysym::XK_KP_Page_Up => VirtualKeyCode::PageUp,
        //x11::keysym::XK_KP_Next => VirtualKeyCode::Kp_next,
        x11::keysym::XK_KP_Page_Down => VirtualKeyCode::PageDown,
        x11::keysym::XK_KP_End => VirtualKeyCode::End,
        //x11::keysym::XK_KP_Begin => VirtualKeyCode::Kp_begin,
        x11::keysym::XK_KP_Insert => VirtualKeyCode::Insert,
        x11::keysym::XK_KP_Delete => VirtualKeyCode::Delete,
        x11::keysym::XK_KP_Equal => VirtualKeyCode::NumpadEquals,
        x11::keysym::XK_KP_Multiply => VirtualKeyCode::NumpadMultiply,
        x11::keysym::XK_KP_Add => VirtualKeyCode::NumpadAdd,
        //x11::keysym::XK_KP_Separator => VirtualKeyCode::Kp_separator,
        x11::keysym::XK_KP_Subtract => VirtualKeyCode::NumpadSubtract,
        x11::keysym::XK_KP_Decimal => VirtualKeyCode::NumpadDecimal,
        x11::keysym::XK_KP_Divide => VirtualKeyCode::NumpadDivide,
        x11::keysym::XK_KP_0 => VirtualKeyCode::Numpad0,
        x11::keysym::XK_KP_1 => VirtualKeyCode::Numpad1,
        x11::keysym::XK_KP_2 => VirtualKeyCode::Numpad2,
        x11::keysym::XK_KP_3 => VirtualKeyCode::Numpad3,
        x11::keysym::XK_KP_4 => VirtualKeyCode::Numpad4,
        x11::keysym::XK_KP_5 => VirtualKeyCode::Numpad5,
        x11::keysym::XK_KP_6 => VirtualKeyCode::Numpad6,
        x11::keysym::XK_KP_7 => VirtualKeyCode::Numpad7,
        x11::keysym::XK_KP_8 => VirtualKeyCode::Numpad8,
        x11::keysym::XK_KP_9 => VirtualKeyCode::Numpad9,
        x11::keysym::XK_F1 => VirtualKeyCode::F1,
        x11::keysym::XK_F2 => VirtualKeyCode::F2,
        x11::keysym::XK_F3 => VirtualKeyCode::F3,
        x11::keysym::XK_F4 => VirtualKeyCode::F4,
        x11::keysym::XK_F5 => VirtualKeyCode::F5,
        x11::keysym::XK_F6 => VirtualKeyCode::F6,
        x11::keysym::XK_F7 => VirtualKeyCode::F7,
        x11::keysym::XK_F8 => VirtualKeyCode::F8,
        x11::keysym::XK_F9 => VirtualKeyCode::F9,
        x11::keysym::XK_F10 => VirtualKeyCode::F10,
        x11::keysym::XK_F11 => VirtualKeyCode::F11,
        //x11::keysym::XK_L1 => VirtualKeyCode::L1,
        x11::keysym::XK_F12 => VirtualKeyCode::F12,
        //x11::keysym::XK_L2 => VirtualKeyCode::L2,
        x11::keysym::XK_F13 => VirtualKeyCode::F13,
        //x11::keysym::XK_L3 => VirtualKeyCode::L3,
        x11::keysym::XK_F14 => VirtualKeyCode::F14,
        //x11::keysym::XK_L4 => VirtualKeyCode::L4,
        x11::keysym::XK_F15 => VirtualKeyCode::F15,
        //x11::keysym::XK_L5 => VirtualKeyCode::L5,
        x11::keysym::XK_F16 => VirtualKeyCode::F16,
        //x11::keysym::XK_L6 => VirtualKeyCode::L6,
        x11::keysym::XK_F17 => VirtualKeyCode::F17,
        //x11::keysym::XK_L7 => VirtualKeyCode::L7,
        x11::keysym::XK_F18 => VirtualKeyCode::F18,
        //x11::keysym::XK_L8 => VirtualKeyCode::L8,
        x11::keysym::XK_F19 => VirtualKeyCode::F19,
        //x11::keysym::XK_L9 => VirtualKeyCode::L9,
        x11::keysym::XK_F20 => VirtualKeyCode::F20,
        //x11::keysym::XK_L10 => VirtualKeyCode::L10,
        x11::keysym::XK_F21 => VirtualKeyCode::F21,
        //x11::keysym::XK_R1 => VirtualKeyCode::R1,
        x11::keysym::XK_F22 => VirtualKeyCode::F22,
        //x11::keysym::XK_R2 => VirtualKeyCode::R2,
        x11::keysym::XK_F23 => VirtualKeyCode::F23,
        //x11::keysym::XK_R3 => VirtualKeyCode::R3,
        x11::keysym::XK_F24 => VirtualKeyCode::F24,
        //x11::keysym::XK_R4 => VirtualKeyCode::R4,
        //x11::keysym::XK_F25 => VirtualKeyCode::F25,
        //x11::keysym::XK_R5 => VirtualKeyCode::R5,
        //x11::keysym::XK_F26 => VirtualKeyCode::F26,
        //x11::keysym::XK_R6 => VirtualKeyCode::R6,
        //x11::keysym::XK_F27 => VirtualKeyCode::F27,
        //x11::keysym::XK_R7 => VirtualKeyCode::R7,
        //x11::keysym::XK_F28 => VirtualKeyCode::F28,
        //x11::keysym::XK_R8 => VirtualKeyCode::R8,
        //x11::keysym::XK_F29 => VirtualKeyCode::F29,
        //x11::keysym::XK_R9 => VirtualKeyCode::R9,
        //x11::keysym::XK_F30 => VirtualKeyCode::F30,
        //x11::keysym::XK_R10 => VirtualKeyCode::R10,
        //x11::keysym::XK_F31 => VirtualKeyCode::F31,
        //x11::keysym::XK_R11 => VirtualKeyCode::R11,
        //x11::keysym::XK_F32 => VirtualKeyCode::F32,
        //x11::keysym::XK_R12 => VirtualKeyCode::R12,
        //x11::keysym::XK_F33 => VirtualKeyCode::F33,
        //x11::keysym::XK_R13 => VirtualKeyCode::R13,
        //x11::keysym::XK_F34 => VirtualKeyCode::F34,
        //x11::keysym::XK_R14 => VirtualKeyCode::R14,
        //x11::keysym::XK_F35 => VirtualKeyCode::F35,
        //x11::keysym::XK_R15 => VirtualKeyCode::R15,
        x11::keysym::XK_Shift_L => VirtualKeyCode::LShift,
        x11::keysym::XK_Shift_R => VirtualKeyCode::RShift,
        x11::keysym::XK_Control_L => VirtualKeyCode::LControl,
        x11::keysym::XK_Control_R => VirtualKeyCode::RControl,
        //x11::keysym::XK_Caps_Lock => VirtualKeyCode::Caps_lock,
        //x11::keysym::XK_Shift_Lock => VirtualKeyCode::Shift_lock,
        //x11::keysym::XK_Meta_L => VirtualKeyCode::Meta_l,
        //x11::keysym::XK_Meta_R => VirtualKeyCode::Meta_r,
        x11::keysym::XK_Alt_L => VirtualKeyCode::LAlt,
        x11::keysym::XK_Alt_R => VirtualKeyCode::RAlt,
        //x11::keysym::XK_Super_L => VirtualKeyCode::Super_l,
        //x11::keysym::XK_Super_R => VirtualKeyCode::Super_r,
        //x11::keysym::XK_Hyper_L => VirtualKeyCode::Hyper_l,
        //x11::keysym::XK_Hyper_R => VirtualKeyCode::Hyper_r,
        x11::keysym::XK_ISO_Left_Tab => VirtualKeyCode::Tab,
        x11::keysym::XK_space => VirtualKeyCode::Space,
        //x11::keysym::XK_exclam => VirtualKeyCode::Exclam,
        //x11::keysym::XK_quotedbl => VirtualKeyCode::Quotedbl,
        //x11::keysym::XK_numbersign => VirtualKeyCode::Numbersign,
        //x11::keysym::XK_dollar => VirtualKeyCode::Dollar,
        //x11::keysym::XK_percent => VirtualKeyCode::Percent,
        //x11::keysym::XK_ampersand => VirtualKeyCode::Ampersand,
        x11::keysym::XK_apostrophe => VirtualKeyCode::Apostrophe,
        //x11::keysym::XK_quoteright => VirtualKeyCode::Quoteright,
        //x11::keysym::XK_parenleft => VirtualKeyCode::Parenleft,
        //x11::keysym::XK_parenright => VirtualKeyCode::Parenright,
        x11::keysym::XK_asterisk => VirtualKeyCode::Asterisk,
        x11::keysym::XK_plus => VirtualKeyCode::Plus,
        x11::keysym::XK_comma => VirtualKeyCode::Comma,
        x11::keysym::XK_minus => VirtualKeyCode::Minus,
        x11::keysym::XK_period => VirtualKeyCode::Period,
        x11::keysym::XK_slash => VirtualKeyCode::Slash,
        x11::keysym::XK_0 => VirtualKeyCode::Zero,
        x11::keysym::XK_1 => VirtualKeyCode::One,
        x11::keysym::XK_2 => VirtualKeyCode::Two,
        x11::keysym::XK_3 => VirtualKeyCode::Three,
        x11::keysym::XK_4 => VirtualKeyCode::Four,
        x11::keysym::XK_5 => VirtualKeyCode::Five,
        x11::keysym::XK_6 => VirtualKeyCode::Six,
        x11::keysym::XK_7 => VirtualKeyCode::Seven,
        x11::keysym::XK_8 => VirtualKeyCode::Eight,
        x11::keysym::XK_9 => VirtualKeyCode::Nine,
        x11::keysym::XK_colon => VirtualKeyCode::Colon,
        x11::keysym::XK_semicolon => VirtualKeyCode::Semicolon,
        //x11::keysym::XK_less => VirtualKeyCode::Less,
        x11::keysym::XK_equal => VirtualKeyCode::Equals,
        //x11::keysym::XK_greater => VirtualKeyCode::Greater,
        //x11::keysym::XK_question => VirtualKeyCode::Question,
        x11::keysym::XK_at => VirtualKeyCode::At,
        x11::keysym::XK_A => VirtualKeyCode::A,
        x11::keysym::XK_B => VirtualKeyCode::B,
        x11::keysym::XK_C => VirtualKeyCode::C,
        x11::keysym::XK_D => VirtualKeyCode::D,
        x11::keysym::XK_E => VirtualKeyCode::E,
        x11::keysym::XK_F => VirtualKeyCode::F,
        x11::keysym::XK_G => VirtualKeyCode::G,
        x11::keysym::XK_H => VirtualKeyCode::H,
        x11::keysym::XK_I => VirtualKeyCode::I,
        x11::keysym::XK_J => VirtualKeyCode::J,
        x11::keysym::XK_K => VirtualKeyCode::K,
        x11::keysym::XK_L => VirtualKeyCode::L,
        x11::keysym::XK_M => VirtualKeyCode::M,
        x11::keysym::XK_N => VirtualKeyCode::N,
        x11::keysym::XK_O => VirtualKeyCode::O,
        x11::keysym::XK_P => VirtualKeyCode::P,
        x11::keysym::XK_Q => VirtualKeyCode::Q,
        x11::keysym::XK_R => VirtualKeyCode::R,
        x11::keysym::XK_S => VirtualKeyCode::S,
        x11::keysym::XK_T => VirtualKeyCode::T,
        x11::keysym::XK_U => VirtualKeyCode::U,
        x11::keysym::XK_V => VirtualKeyCode::V,
        x11::keysym::XK_W => VirtualKeyCode::W,
        x11::keysym::XK_X => VirtualKeyCode::X,
        x11::keysym::XK_Y => VirtualKeyCode::Y,
        x11::keysym::XK_Z => VirtualKeyCode::Z,
        x11::keysym::XK_bracketleft => VirtualKeyCode::LBracket,
        x11::keysym::XK_backslash => VirtualKeyCode::Backslash,
        x11::keysym::XK_bracketright => VirtualKeyCode::RBracket,
        //x11::keysym::XK_asciicircum => VirtualKeyCode::Asciicircum,
        //x11::keysym::XK_underscore => VirtualKeyCode::Underscore,
        x11::keysym::XK_grave => VirtualKeyCode::Grave,
        //x11::keysym::XK_quoteleft => VirtualKeyCode::Quoteleft,
        x11::keysym::XK_a => VirtualKeyCode::A,
        x11::keysym::XK_b => VirtualKeyCode::B,
        x11::keysym::XK_c => VirtualKeyCode::C,
        x11::keysym::XK_d => VirtualKeyCode::D,
        x11::keysym::XK_e => VirtualKeyCode::E,
        x11::keysym::XK_f => VirtualKeyCode::F,
        x11::keysym::XK_g => VirtualKeyCode::G,
        x11::keysym::XK_h => VirtualKeyCode::H,
        x11::keysym::XK_i => VirtualKeyCode::I,
        x11::keysym::XK_j => VirtualKeyCode::J,
        x11::keysym::XK_k => VirtualKeyCode::K,
        x11::keysym::XK_l => VirtualKeyCode::L,
        x11::keysym::XK_m => VirtualKeyCode::M,
        x11::keysym::XK_n => VirtualKeyCode::N,
        x11::keysym::XK_o => VirtualKeyCode::O,
        x11::keysym::XK_p => VirtualKeyCode::P,
        x11::keysym::XK_q => VirtualKeyCode::Q,
        x11::keysym::XK_r => VirtualKeyCode::R,
        x11::keysym::XK_s => VirtualKeyCode::S,
        x11::keysym::XK_t => VirtualKeyCode::T,
        x11::keysym::XK_u => VirtualKeyCode::U,
        x11::keysym::XK_v => VirtualKeyCode::V,
        x11::keysym::XK_w => VirtualKeyCode::W,
        x11::keysym::XK_x => VirtualKeyCode::X,
        x11::keysym::XK_y => VirtualKeyCode::Y,
        x11::keysym::XK_z => VirtualKeyCode::Z,
        //x11::keysym::XK_braceleft => VirtualKeyCode::Braceleft,
        //x11::keysym::XK_bar => VirtualKeyCode::Bar,
        //x11::keysym::XK_braceright => VirtualKeyCode::Braceright,
        //x11::keysym::XK_asciitilde => VirtualKeyCode::Asciitilde,
        //x11::keysym::XK_nobreakspace => VirtualKeyCode::Nobreakspace,
        //x11::keysym::XK_exclamdown => VirtualKeyCode::Exclamdown,
        //x11::keysym::XK_cent => VirtualKeyCode::Cent,
        //x11::keysym::XK_sterling => VirtualKeyCode::Sterling,
        //x11::keysym::XK_currency => VirtualKeyCode::Currency,
        //x11::keysym::XK_yen => VirtualKeyCode::Yen,
        //x11::keysym::XK_brokenbar => VirtualKeyCode::Brokenbar,
        //x11::keysym::XK_section => VirtualKeyCode::Section,
        //x11::keysym::XK_diaeresis => VirtualKeyCode::Diaeresis,
        //x11::keysym::XK_copyright => VirtualKeyCode::Copyright,
        //x11::keysym::XK_ordfeminine => VirtualKeyCode::Ordfeminine,
        //x11::keysym::XK_guillemotleft => VirtualKeyCode::Guillemotleft,
        //x11::keysym::XK_notsign => VirtualKeyCode::Notsign,
        //x11::keysym::XK_hyphen => VirtualKeyCode::Hyphen,
        //x11::keysym::XK_registered => VirtualKeyCode::Registered,
        //x11::keysym::XK_macron => VirtualKeyCode::Macron,
        //x11::keysym::XK_degree => VirtualKeyCode::Degree,
        //x11::keysym::XK_plusminus => VirtualKeyCode::Plusminus,
        //x11::keysym::XK_twosuperior => VirtualKeyCode::Twosuperior,
        //x11::keysym::XK_threesuperior => VirtualKeyCode::Threesuperior,
        //x11::keysym::XK_acute => VirtualKeyCode::Acute,
        //x11::keysym::XK_mu => VirtualKeyCode::Mu,
        //x11::keysym::XK_paragraph => VirtualKeyCode::Paragraph,
        //x11::keysym::XK_periodcentered => VirtualKeyCode::Periodcentered,
        //x11::keysym::XK_cedilla => VirtualKeyCode::Cedilla,
        //x11::keysym::XK_onesuperior => VirtualKeyCode::Onesuperior,
        //x11::keysym::XK_masculine => VirtualKeyCode::Masculine,
        //x11::keysym::XK_guillemotright => VirtualKeyCode::Guillemotright,
        //x11::keysym::XK_onequarter => VirtualKeyCode::Onequarter,
        //x11::keysym::XK_onehalf => VirtualKeyCode::Onehalf,
        //x11::keysym::XK_threequarters => VirtualKeyCode::Threequarters,
        //x11::keysym::XK_questiondown => VirtualKeyCode::Questiondown,
        //x11::keysym::XK_Agrave => VirtualKeyCode::Agrave,
        //x11::keysym::XK_Aacute => VirtualKeyCode::Aacute,
        //x11::keysym::XK_Acircumflex => VirtualKeyCode::Acircumflex,
        //x11::keysym::XK_Atilde => VirtualKeyCode::Atilde,
        //x11::keysym::XK_Adiaeresis => VirtualKeyCode::Adiaeresis,
        //x11::keysym::XK_Aring => VirtualKeyCode::Aring,
        //x11::keysym::XK_AE => VirtualKeyCode::Ae,
        //x11::keysym::XK_Ccedilla => VirtualKeyCode::Ccedilla,
        //x11::keysym::XK_Egrave => VirtualKeyCode::Egrave,
        //x11::keysym::XK_Eacute => VirtualKeyCode::Eacute,
        //x11::keysym::XK_Ecircumflex => VirtualKeyCode::Ecircumflex,
        //x11::keysym::XK_Ediaeresis => VirtualKeyCode::Ediaeresis,
        //x11::keysym::XK_Igrave => VirtualKeyCode::Igrave,
        //x11::keysym::XK_Iacute => VirtualKeyCode::Iacute,
        //x11::keysym::XK_Icircumflex => VirtualKeyCode::Icircumflex,
        //x11::keysym::XK_Idiaeresis => VirtualKeyCode::Idiaeresis,
        //x11::keysym::XK_ETH => VirtualKeyCode::Eth,
        //x11::keysym::XK_Eth => VirtualKeyCode::Eth,
        //x11::keysym::XK_Ntilde => VirtualKeyCode::Ntilde,
        //x11::keysym::XK_Ograve => VirtualKeyCode::Ograve,
        //x11::keysym::XK_Oacute => VirtualKeyCode::Oacute,
        //x11::keysym::XK_Ocircumflex => VirtualKeyCode::Ocircumflex,
        //x11::keysym::XK_Otilde => VirtualKeyCode::Otilde,
        //x11::keysym::XK_Odiaeresis => VirtualKeyCode::Odiaeresis,
        //x11::keysym::XK_multiply => VirtualKeyCode::Multiply,
        //x11::keysym::XK_Ooblique => VirtualKeyCode::Ooblique,
        //x11::keysym::XK_Ugrave => VirtualKeyCode::Ugrave,
        //x11::keysym::XK_Uacute => VirtualKeyCode::Uacute,
        //x11::keysym::XK_Ucircumflex => VirtualKeyCode::Ucircumflex,
        //x11::keysym::XK_Udiaeresis => VirtualKeyCode::Udiaeresis,
        //x11::keysym::XK_Yacute => VirtualKeyCode::Yacute,
        //x11::keysym::XK_THORN => VirtualKeyCode::Thorn,
        //x11::keysym::XK_Thorn => VirtualKeyCode::Thorn,
        //x11::keysym::XK_ssharp => VirtualKeyCode::Ssharp,
        //x11::keysym::XK_agrave => VirtualKeyCode::Agrave,
        //x11::keysym::XK_aacute => VirtualKeyCode::Aacute,
        //x11::keysym::XK_acircumflex => VirtualKeyCode::Acircumflex,
        //x11::keysym::XK_atilde => VirtualKeyCode::Atilde,
        //x11::keysym::XK_adiaeresis => VirtualKeyCode::Adiaeresis,
        //x11::keysym::XK_aring => VirtualKeyCode::Aring,
        //x11::keysym::XK_ae => VirtualKeyCode::Ae,
        //x11::keysym::XK_ccedilla => VirtualKeyCode::Ccedilla,
        //x11::keysym::XK_egrave => VirtualKeyCode::Egrave,
        //x11::keysym::XK_eacute => VirtualKeyCode::Eacute,
        //x11::keysym::XK_ecircumflex => VirtualKeyCode::Ecircumflex,
        //x11::keysym::XK_ediaeresis => VirtualKeyCode::Ediaeresis,
        //x11::keysym::XK_igrave => VirtualKeyCode::Igrave,
        //x11::keysym::XK_iacute => VirtualKeyCode::Iacute,
        //x11::keysym::XK_icircumflex => VirtualKeyCode::Icircumflex,
        //x11::keysym::XK_idiaeresis => VirtualKeyCode::Idiaeresis,
        //x11::keysym::XK_eth => VirtualKeyCode::Eth,
        //x11::keysym::XK_ntilde => VirtualKeyCode::Ntilde,
        //x11::keysym::XK_ograve => VirtualKeyCode::Ograve,
        //x11::keysym::XK_oacute => VirtualKeyCode::Oacute,
        //x11::keysym::XK_ocircumflex => VirtualKeyCode::Ocircumflex,
        //x11::keysym::XK_otilde => VirtualKeyCode::Otilde,
        //x11::keysym::XK_odiaeresis => VirtualKeyCode::Odiaeresis,
        //x11::keysym::XK_division => VirtualKeyCode::Division,
        //x11::keysym::XK_oslash => VirtualKeyCode::Oslash,
        //x11::keysym::XK_ugrave => VirtualKeyCode::Ugrave,
        //x11::keysym::XK_uacute => VirtualKeyCode::Uacute,
        //x11::keysym::XK_ucircumflex => VirtualKeyCode::Ucircumflex,
        //x11::keysym::XK_udiaeresis => VirtualKeyCode::Udiaeresis,
        //x11::keysym::XK_yacute => VirtualKeyCode::Yacute,
        //x11::keysym::XK_thorn => VirtualKeyCode::Thorn,
        //x11::keysym::XK_ydiaeresis => VirtualKeyCode::Ydiaeresis,
        //x11::keysym::XK_Aogonek => VirtualKeyCode::Aogonek,
        //x11::keysym::XK_breve => VirtualKeyCode::Breve,
        //x11::keysym::XK_Lstroke => VirtualKeyCode::Lstroke,
        //x11::keysym::XK_Lcaron => VirtualKeyCode::Lcaron,
        //x11::keysym::XK_Sacute => VirtualKeyCode::Sacute,
        //x11::keysym::XK_Scaron => VirtualKeyCode::Scaron,
        //x11::keysym::XK_Scedilla => VirtualKeyCode::Scedilla,
        //x11::keysym::XK_Tcaron => VirtualKeyCode::Tcaron,
        //x11::keysym::XK_Zacute => VirtualKeyCode::Zacute,
        //x11::keysym::XK_Zcaron => VirtualKeyCode::Zcaron,
        //x11::keysym::XK_Zabovedot => VirtualKeyCode::Zabovedot,
        //x11::keysym::XK_aogonek => VirtualKeyCode::Aogonek,
        //x11::keysym::XK_ogonek => VirtualKeyCode::Ogonek,
        //x11::keysym::XK_lstroke => VirtualKeyCode::Lstroke,
        //x11::keysym::XK_lcaron => VirtualKeyCode::Lcaron,
        //x11::keysym::XK_sacute => VirtualKeyCode::Sacute,
        //x11::keysym::XK_caron => VirtualKeyCode::Caron,
        //x11::keysym::XK_scaron => VirtualKeyCode::Scaron,
        //x11::keysym::XK_scedilla => VirtualKeyCode::Scedilla,
        //x11::keysym::XK_tcaron => VirtualKeyCode::Tcaron,
        //x11::keysym::XK_zacute => VirtualKeyCode::Zacute,
        //x11::keysym::XK_doubleacute => VirtualKeyCode::Doubleacute,
        //x11::keysym::XK_zcaron => VirtualKeyCode::Zcaron,
        //x11::keysym::XK_zabovedot => VirtualKeyCode::Zabovedot,
        //x11::keysym::XK_Racute => VirtualKeyCode::Racute,
        //x11::keysym::XK_Abreve => VirtualKeyCode::Abreve,
        //x11::keysym::XK_Lacute => VirtualKeyCode::Lacute,
        //x11::keysym::XK_Cacute => VirtualKeyCode::Cacute,
        //x11::keysym::XK_Ccaron => VirtualKeyCode::Ccaron,
        //x11::keysym::XK_Eogonek => VirtualKeyCode::Eogonek,
        //x11::keysym::XK_Ecaron => VirtualKeyCode::Ecaron,
        //x11::keysym::XK_Dcaron => VirtualKeyCode::Dcaron,
        //x11::keysym::XK_Dstroke => VirtualKeyCode::Dstroke,
        //x11::keysym::XK_Nacute => VirtualKeyCode::Nacute,
        //x11::keysym::XK_Ncaron => VirtualKeyCode::Ncaron,
        //x11::keysym::XK_Odoubleacute => VirtualKeyCode::Odoubleacute,
        //x11::keysym::XK_Rcaron => VirtualKeyCode::Rcaron,
        //x11::keysym::XK_Uring => VirtualKeyCode::Uring,
        //x11::keysym::XK_Udoubleacute => VirtualKeyCode::Udoubleacute,
        //x11::keysym::XK_Tcedilla => VirtualKeyCode::Tcedilla,
        //x11::keysym::XK_racute => VirtualKeyCode::Racute,
        //x11::keysym::XK_abreve => VirtualKeyCode::Abreve,
        //x11::keysym::XK_lacute => VirtualKeyCode::Lacute,
        //x11::keysym::XK_cacute => VirtualKeyCode::Cacute,
        //x11::keysym::XK_ccaron => VirtualKeyCode::Ccaron,
        //x11::keysym::XK_eogonek => VirtualKeyCode::Eogonek,
        //x11::keysym::XK_ecaron => VirtualKeyCode::Ecaron,
        //x11::keysym::XK_dcaron => VirtualKeyCode::Dcaron,
        //x11::keysym::XK_dstroke => VirtualKeyCode::Dstroke,
        //x11::keysym::XK_nacute => VirtualKeyCode::Nacute,
        //x11::keysym::XK_ncaron => VirtualKeyCode::Ncaron,
        //x11::keysym::XK_odoubleacute => VirtualKeyCode::Odoubleacute,
        //x11::keysym::XK_udoubleacute => VirtualKeyCode::Udoubleacute,
        //x11::keysym::XK_rcaron => VirtualKeyCode::Rcaron,
        //x11::keysym::XK_uring => VirtualKeyCode::Uring,
        //x11::keysym::XK_tcedilla => VirtualKeyCode::Tcedilla,
        //x11::keysym::XK_abovedot => VirtualKeyCode::Abovedot,
        //x11::keysym::XK_Hstroke => VirtualKeyCode::Hstroke,
        //x11::keysym::XK_Hcircumflex => VirtualKeyCode::Hcircumflex,
        //x11::keysym::XK_Iabovedot => VirtualKeyCode::Iabovedot,
        //x11::keysym::XK_Gbreve => VirtualKeyCode::Gbreve,
        //x11::keysym::XK_Jcircumflex => VirtualKeyCode::Jcircumflex,
        //x11::keysym::XK_hstroke => VirtualKeyCode::Hstroke,
        //x11::keysym::XK_hcircumflex => VirtualKeyCode::Hcircumflex,
        //x11::keysym::XK_idotless => VirtualKeyCode::Idotless,
        //x11::keysym::XK_gbreve => VirtualKeyCode::Gbreve,
        //x11::keysym::XK_jcircumflex => VirtualKeyCode::Jcircumflex,
        //x11::keysym::XK_Cabovedot => VirtualKeyCode::Cabovedot,
        //x11::keysym::XK_Ccircumflex => VirtualKeyCode::Ccircumflex,
        //x11::keysym::XK_Gabovedot => VirtualKeyCode::Gabovedot,
        //x11::keysym::XK_Gcircumflex => VirtualKeyCode::Gcircumflex,
        //x11::keysym::XK_Ubreve => VirtualKeyCode::Ubreve,
        //x11::keysym::XK_Scircumflex => VirtualKeyCode::Scircumflex,
        //x11::keysym::XK_cabovedot => VirtualKeyCode::Cabovedot,
        //x11::keysym::XK_ccircumflex => VirtualKeyCode::Ccircumflex,
        //x11::keysym::XK_gabovedot => VirtualKeyCode::Gabovedot,
        //x11::keysym::XK_gcircumflex => VirtualKeyCode::Gcircumflex,
        //x11::keysym::XK_ubreve => VirtualKeyCode::Ubreve,
        //x11::keysym::XK_scircumflex => VirtualKeyCode::Scircumflex,
        //x11::keysym::XK_kra => VirtualKeyCode::Kra,
        //x11::keysym::XK_kappa => VirtualKeyCode::Kappa,
        //x11::keysym::XK_Rcedilla => VirtualKeyCode::Rcedilla,
        //x11::keysym::XK_Itilde => VirtualKeyCode::Itilde,
        //x11::keysym::XK_Lcedilla => VirtualKeyCode::Lcedilla,
        //x11::keysym::XK_Emacron => VirtualKeyCode::Emacron,
        //x11::keysym::XK_Gcedilla => VirtualKeyCode::Gcedilla,
        //x11::keysym::XK_Tslash => VirtualKeyCode::Tslash,
        //x11::keysym::XK_rcedilla => VirtualKeyCode::Rcedilla,
        //x11::keysym::XK_itilde => VirtualKeyCode::Itilde,
        //x11::keysym::XK_lcedilla => VirtualKeyCode::Lcedilla,
        //x11::keysym::XK_emacron => VirtualKeyCode::Emacron,
        //x11::keysym::XK_gcedilla => VirtualKeyCode::Gcedilla,
        //x11::keysym::XK_tslash => VirtualKeyCode::Tslash,
        //x11::keysym::XK_ENG => VirtualKeyCode::Eng,
        //x11::keysym::XK_eng => VirtualKeyCode::Eng,
        //x11::keysym::XK_Amacron => VirtualKeyCode::Amacron,
        //x11::keysym::XK_Iogonek => VirtualKeyCode::Iogonek,
        //x11::keysym::XK_Eabovedot => VirtualKeyCode::Eabovedot,
        //x11::keysym::XK_Imacron => VirtualKeyCode::Imacron,
        //x11::keysym::XK_Ncedilla => VirtualKeyCode::Ncedilla,
        //x11::keysym::XK_Omacron => VirtualKeyCode::Omacron,
        //x11::keysym::XK_Kcedilla => VirtualKeyCode::Kcedilla,
        //x11::keysym::XK_Uogonek => VirtualKeyCode::Uogonek,
        //x11::keysym::XK_Utilde => VirtualKeyCode::Utilde,
        //x11::keysym::XK_Umacron => VirtualKeyCode::Umacron,
        //x11::keysym::XK_amacron => VirtualKeyCode::Amacron,
        //x11::keysym::XK_iogonek => VirtualKeyCode::Iogonek,
        //x11::keysym::XK_eabovedot => VirtualKeyCode::Eabovedot,
        //x11::keysym::XK_imacron => VirtualKeyCode::Imacron,
        //x11::keysym::XK_ncedilla => VirtualKeyCode::Ncedilla,
        //x11::keysym::XK_omacron => VirtualKeyCode::Omacron,
        //x11::keysym::XK_kcedilla => VirtualKeyCode::Kcedilla,
        //x11::keysym::XK_uogonek => VirtualKeyCode::Uogonek,
        //x11::keysym::XK_utilde => VirtualKeyCode::Utilde,
        //x11::keysym::XK_umacron => VirtualKeyCode::Umacron,
        //x11::keysym::XK_overline => VirtualKeyCode::Overline,
        //x11::keysym::XK_kana_fullstop => VirtualKeyCode::Kana_fullstop,
        //x11::keysym::XK_kana_openingbracket => VirtualKeyCode::Kana_openingbracket,
        //x11::keysym::XK_kana_closingbracket => VirtualKeyCode::Kana_closingbracket,
        //x11::keysym::XK_kana_comma => VirtualKeyCode::Kana_comma,
        //x11::keysym::XK_kana_conjunctive => VirtualKeyCode::Kana_conjunctive,
        //x11::keysym::XK_kana_middledot => VirtualKeyCode::Kana_middledot,
        //x11::keysym::XK_kana_WO => VirtualKeyCode::Kana_wo,
        //x11::keysym::XK_kana_a => VirtualKeyCode::Kana_a,
        //x11::keysym::XK_kana_i => VirtualKeyCode::Kana_i,
        //x11::keysym::XK_kana_u => VirtualKeyCode::Kana_u,
        //x11::keysym::XK_kana_e => VirtualKeyCode::Kana_e,
        //x11::keysym::XK_kana_o => VirtualKeyCode::Kana_o,
        //x11::keysym::XK_kana_ya => VirtualKeyCode::Kana_ya,
        //x11::keysym::XK_kana_yu => VirtualKeyCode::Kana_yu,
        //x11::keysym::XK_kana_yo => VirtualKeyCode::Kana_yo,
        //x11::keysym::XK_kana_tsu => VirtualKeyCode::Kana_tsu,
        //x11::keysym::XK_kana_tu => VirtualKeyCode::Kana_tu,
        //x11::keysym::XK_prolongedsound => VirtualKeyCode::Prolongedsound,
        //x11::keysym::XK_kana_A => VirtualKeyCode::Kana_a,
        //x11::keysym::XK_kana_I => VirtualKeyCode::Kana_i,
        //x11::keysym::XK_kana_U => VirtualKeyCode::Kana_u,
        //x11::keysym::XK_kana_E => VirtualKeyCode::Kana_e,
        //x11::keysym::XK_kana_O => VirtualKeyCode::Kana_o,
        //x11::keysym::XK_kana_KA => VirtualKeyCode::Kana_ka,
        //x11::keysym::XK_kana_KI => VirtualKeyCode::Kana_ki,
        //x11::keysym::XK_kana_KU => VirtualKeyCode::Kana_ku,
        //x11::keysym::XK_kana_KE => VirtualKeyCode::Kana_ke,
        //x11::keysym::XK_kana_KO => VirtualKeyCode::Kana_ko,
        //x11::keysym::XK_kana_SA => VirtualKeyCode::Kana_sa,
        //x11::keysym::XK_kana_SHI => VirtualKeyCode::Kana_shi,
        //x11::keysym::XK_kana_SU => VirtualKeyCode::Kana_su,
        //x11::keysym::XK_kana_SE => VirtualKeyCode::Kana_se,
        //x11::keysym::XK_kana_SO => VirtualKeyCode::Kana_so,
        //x11::keysym::XK_kana_TA => VirtualKeyCode::Kana_ta,
        //x11::keysym::XK_kana_CHI => VirtualKeyCode::Kana_chi,
        //x11::keysym::XK_kana_TI => VirtualKeyCode::Kana_ti,
        //x11::keysym::XK_kana_TSU => VirtualKeyCode::Kana_tsu,
        //x11::keysym::XK_kana_TU => VirtualKeyCode::Kana_tu,
        //x11::keysym::XK_kana_TE => VirtualKeyCode::Kana_te,
        //x11::keysym::XK_kana_TO => VirtualKeyCode::Kana_to,
        //x11::keysym::XK_kana_NA => VirtualKeyCode::Kana_na,
        //x11::keysym::XK_kana_NI => VirtualKeyCode::Kana_ni,
        //x11::keysym::XK_kana_NU => VirtualKeyCode::Kana_nu,
        //x11::keysym::XK_kana_NE => VirtualKeyCode::Kana_ne,
        //x11::keysym::XK_kana_NO => VirtualKeyCode::Kana_no,
        //x11::keysym::XK_kana_HA => VirtualKeyCode::Kana_ha,
        //x11::keysym::XK_kana_HI => VirtualKeyCode::Kana_hi,
        //x11::keysym::XK_kana_FU => VirtualKeyCode::Kana_fu,
        //x11::keysym::XK_kana_HU => VirtualKeyCode::Kana_hu,
        //x11::keysym::XK_kana_HE => VirtualKeyCode::Kana_he,
        //x11::keysym::XK_kana_HO => VirtualKeyCode::Kana_ho,
        //x11::keysym::XK_kana_MA => VirtualKeyCode::Kana_ma,
        //x11::keysym::XK_kana_MI => VirtualKeyCode::Kana_mi,
        //x11::keysym::XK_kana_MU => VirtualKeyCode::Kana_mu,
        //x11::keysym::XK_kana_ME => VirtualKeyCode::Kana_me,
        //x11::keysym::XK_kana_MO => VirtualKeyCode::Kana_mo,
        //x11::keysym::XK_kana_YA => VirtualKeyCode::Kana_ya,
        //x11::keysym::XK_kana_YU => VirtualKeyCode::Kana_yu,
        //x11::keysym::XK_kana_YO => VirtualKeyCode::Kana_yo,
        //x11::keysym::XK_kana_RA => VirtualKeyCode::Kana_ra,
        //x11::keysym::XK_kana_RI => VirtualKeyCode::Kana_ri,
        //x11::keysym::XK_kana_RU => VirtualKeyCode::Kana_ru,
        //x11::keysym::XK_kana_RE => VirtualKeyCode::Kana_re,
        //x11::keysym::XK_kana_RO => VirtualKeyCode::Kana_ro,
        //x11::keysym::XK_kana_WA => VirtualKeyCode::Kana_wa,
        //x11::keysym::XK_kana_N => VirtualKeyCode::Kana_n,
        //x11::keysym::XK_voicedsound => VirtualKeyCode::Voicedsound,
        //x11::keysym::XK_semivoicedsound => VirtualKeyCode::Semivoicedsound,
        //x11::keysym::XK_kana_switch => VirtualKeyCode::Kana_switch,
        //x11::keysym::XK_Arabic_comma => VirtualKeyCode::Arabic_comma,
        //x11::keysym::XK_Arabic_semicolon => VirtualKeyCode::Arabic_semicolon,
        //x11::keysym::XK_Arabic_question_mark => VirtualKeyCode::Arabic_question_mark,
        //x11::keysym::XK_Arabic_hamza => VirtualKeyCode::Arabic_hamza,
        //x11::keysym::XK_Arabic_maddaonalef => VirtualKeyCode::Arabic_maddaonalef,
        //x11::keysym::XK_Arabic_hamzaonalef => VirtualKeyCode::Arabic_hamzaonalef,
        //x11::keysym::XK_Arabic_hamzaonwaw => VirtualKeyCode::Arabic_hamzaonwaw,
        //x11::keysym::XK_Arabic_hamzaunderalef => VirtualKeyCode::Arabic_hamzaunderalef,
        //x11::keysym::XK_Arabic_hamzaonyeh => VirtualKeyCode::Arabic_hamzaonyeh,
        //x11::keysym::XK_Arabic_alef => VirtualKeyCode::Arabic_alef,
        //x11::keysym::XK_Arabic_beh => VirtualKeyCode::Arabic_beh,
        //x11::keysym::XK_Arabic_tehmarbuta => VirtualKeyCode::Arabic_tehmarbuta,
        //x11::keysym::XK_Arabic_teh => VirtualKeyCode::Arabic_teh,
        //x11::keysym::XK_Arabic_theh => VirtualKeyCode::Arabic_theh,
        //x11::keysym::XK_Arabic_jeem => VirtualKeyCode::Arabic_jeem,
        //x11::keysym::XK_Arabic_hah => VirtualKeyCode::Arabic_hah,
        //x11::keysym::XK_Arabic_khah => VirtualKeyCode::Arabic_khah,
        //x11::keysym::XK_Arabic_dal => VirtualKeyCode::Arabic_dal,
        //x11::keysym::XK_Arabic_thal => VirtualKeyCode::Arabic_thal,
        //x11::keysym::XK_Arabic_ra => VirtualKeyCode::Arabic_ra,
        //x11::keysym::XK_Arabic_zain => VirtualKeyCode::Arabic_zain,
        //x11::keysym::XK_Arabic_seen => VirtualKeyCode::Arabic_seen,
        //x11::keysym::XK_Arabic_sheen => VirtualKeyCode::Arabic_sheen,
        //x11::keysym::XK_Arabic_sad => VirtualKeyCode::Arabic_sad,
        //x11::keysym::XK_Arabic_dad => VirtualKeyCode::Arabic_dad,
        //x11::keysym::XK_Arabic_tah => VirtualKeyCode::Arabic_tah,
        //x11::keysym::XK_Arabic_zah => VirtualKeyCode::Arabic_zah,
        //x11::keysym::XK_Arabic_ain => VirtualKeyCode::Arabic_ain,
        //x11::keysym::XK_Arabic_ghain => VirtualKeyCode::Arabic_ghain,
        //x11::keysym::XK_Arabic_tatweel => VirtualKeyCode::Arabic_tatweel,
        //x11::keysym::XK_Arabic_feh => VirtualKeyCode::Arabic_feh,
        //x11::keysym::XK_Arabic_qaf => VirtualKeyCode::Arabic_qaf,
        //x11::keysym::XK_Arabic_kaf => VirtualKeyCode::Arabic_kaf,
        //x11::keysym::XK_Arabic_lam => VirtualKeyCode::Arabic_lam,
        //x11::keysym::XK_Arabic_meem => VirtualKeyCode::Arabic_meem,
        //x11::keysym::XK_Arabic_noon => VirtualKeyCode::Arabic_noon,
        //x11::keysym::XK_Arabic_ha => VirtualKeyCode::Arabic_ha,
        //x11::keysym::XK_Arabic_heh => VirtualKeyCode::Arabic_heh,
        //x11::keysym::XK_Arabic_waw => VirtualKeyCode::Arabic_waw,
        //x11::keysym::XK_Arabic_alefmaksura => VirtualKeyCode::Arabic_alefmaksura,
        //x11::keysym::XK_Arabic_yeh => VirtualKeyCode::Arabic_yeh,
        //x11::keysym::XK_Arabic_fathatan => VirtualKeyCode::Arabic_fathatan,
        //x11::keysym::XK_Arabic_dammatan => VirtualKeyCode::Arabic_dammatan,
        //x11::keysym::XK_Arabic_kasratan => VirtualKeyCode::Arabic_kasratan,
        //x11::keysym::XK_Arabic_fatha => VirtualKeyCode::Arabic_fatha,
        //x11::keysym::XK_Arabic_damma => VirtualKeyCode::Arabic_damma,
        //x11::keysym::XK_Arabic_kasra => VirtualKeyCode::Arabic_kasra,
        //x11::keysym::XK_Arabic_shadda => VirtualKeyCode::Arabic_shadda,
        //x11::keysym::XK_Arabic_sukun => VirtualKeyCode::Arabic_sukun,
        //x11::keysym::XK_Arabic_switch => VirtualKeyCode::Arabic_switch,
        //x11::keysym::XK_Serbian_dje => VirtualKeyCode::Serbian_dje,
        //x11::keysym::XK_Macedonia_gje => VirtualKeyCode::Macedonia_gje,
        //x11::keysym::XK_Cyrillic_io => VirtualKeyCode::Cyrillic_io,
        //x11::keysym::XK_Ukrainian_ie => VirtualKeyCode::Ukrainian_ie,
        //x11::keysym::XK_Ukranian_je => VirtualKeyCode::Ukranian_je,
        //x11::keysym::XK_Macedonia_dse => VirtualKeyCode::Macedonia_dse,
        //x11::keysym::XK_Ukrainian_i => VirtualKeyCode::Ukrainian_i,
        //x11::keysym::XK_Ukranian_i => VirtualKeyCode::Ukranian_i,
        //x11::keysym::XK_Ukrainian_yi => VirtualKeyCode::Ukrainian_yi,
        //x11::keysym::XK_Ukranian_yi => VirtualKeyCode::Ukranian_yi,
        //x11::keysym::XK_Cyrillic_je => VirtualKeyCode::Cyrillic_je,
        //x11::keysym::XK_Serbian_je => VirtualKeyCode::Serbian_je,
        //x11::keysym::XK_Cyrillic_lje => VirtualKeyCode::Cyrillic_lje,
        //x11::keysym::XK_Serbian_lje => VirtualKeyCode::Serbian_lje,
        //x11::keysym::XK_Cyrillic_nje => VirtualKeyCode::Cyrillic_nje,
        //x11::keysym::XK_Serbian_nje => VirtualKeyCode::Serbian_nje,
        //x11::keysym::XK_Serbian_tshe => VirtualKeyCode::Serbian_tshe,
        //x11::keysym::XK_Macedonia_kje => VirtualKeyCode::Macedonia_kje,
        //x11::keysym::XK_Byelorussian_shortu => VirtualKeyCode::Byelorussian_shortu,
        //x11::keysym::XK_Cyrillic_dzhe => VirtualKeyCode::Cyrillic_dzhe,
        //x11::keysym::XK_Serbian_dze => VirtualKeyCode::Serbian_dze,
        //x11::keysym::XK_numerosign => VirtualKeyCode::Numerosign,
        //x11::keysym::XK_Serbian_DJE => VirtualKeyCode::Serbian_dje,
        //x11::keysym::XK_Macedonia_GJE => VirtualKeyCode::Macedonia_gje,
        //x11::keysym::XK_Cyrillic_IO => VirtualKeyCode::Cyrillic_io,
        //x11::keysym::XK_Ukrainian_IE => VirtualKeyCode::Ukrainian_ie,
        //x11::keysym::XK_Ukranian_JE => VirtualKeyCode::Ukranian_je,
        //x11::keysym::XK_Macedonia_DSE => VirtualKeyCode::Macedonia_dse,
        //x11::keysym::XK_Ukrainian_I => VirtualKeyCode::Ukrainian_i,
        //x11::keysym::XK_Ukranian_I => VirtualKeyCode::Ukranian_i,
        //x11::keysym::XK_Ukrainian_YI => VirtualKeyCode::Ukrainian_yi,
        //x11::keysym::XK_Ukranian_YI => VirtualKeyCode::Ukranian_yi,
        //x11::keysym::XK_Cyrillic_JE => VirtualKeyCode::Cyrillic_je,
        //x11::keysym::XK_Serbian_JE => VirtualKeyCode::Serbian_je,
        //x11::keysym::XK_Cyrillic_LJE => VirtualKeyCode::Cyrillic_lje,
        //x11::keysym::XK_Serbian_LJE => VirtualKeyCode::Serbian_lje,
        //x11::keysym::XK_Cyrillic_NJE => VirtualKeyCode::Cyrillic_nje,
        //x11::keysym::XK_Serbian_NJE => VirtualKeyCode::Serbian_nje,
        //x11::keysym::XK_Serbian_TSHE => VirtualKeyCode::Serbian_tshe,
        //x11::keysym::XK_Macedonia_KJE => VirtualKeyCode::Macedonia_kje,
        //x11::keysym::XK_Byelorussian_SHORTU => VirtualKeyCode::Byelorussian_shortu,
        //x11::keysym::XK_Cyrillic_DZHE => VirtualKeyCode::Cyrillic_dzhe,
        //x11::keysym::XK_Serbian_DZE => VirtualKeyCode::Serbian_dze,
        //x11::keysym::XK_Cyrillic_yu => VirtualKeyCode::Cyrillic_yu,
        //x11::keysym::XK_Cyrillic_a => VirtualKeyCode::Cyrillic_a,
        //x11::keysym::XK_Cyrillic_be => VirtualKeyCode::Cyrillic_be,
        //x11::keysym::XK_Cyrillic_tse => VirtualKeyCode::Cyrillic_tse,
        //x11::keysym::XK_Cyrillic_de => VirtualKeyCode::Cyrillic_de,
        //x11::keysym::XK_Cyrillic_ie => VirtualKeyCode::Cyrillic_ie,
        //x11::keysym::XK_Cyrillic_ef => VirtualKeyCode::Cyrillic_ef,
        //x11::keysym::XK_Cyrillic_ghe => VirtualKeyCode::Cyrillic_ghe,
        //x11::keysym::XK_Cyrillic_ha => VirtualKeyCode::Cyrillic_ha,
        //x11::keysym::XK_Cyrillic_i => VirtualKeyCode::Cyrillic_i,
        //x11::keysym::XK_Cyrillic_shorti => VirtualKeyCode::Cyrillic_shorti,
        //x11::keysym::XK_Cyrillic_ka => VirtualKeyCode::Cyrillic_ka,
        //x11::keysym::XK_Cyrillic_el => VirtualKeyCode::Cyrillic_el,
        //x11::keysym::XK_Cyrillic_em => VirtualKeyCode::Cyrillic_em,
        //x11::keysym::XK_Cyrillic_en => VirtualKeyCode::Cyrillic_en,
        //x11::keysym::XK_Cyrillic_o => VirtualKeyCode::Cyrillic_o,
        //x11::keysym::XK_Cyrillic_pe => VirtualKeyCode::Cyrillic_pe,
        //x11::keysym::XK_Cyrillic_ya => VirtualKeyCode::Cyrillic_ya,
        //x11::keysym::XK_Cyrillic_er => VirtualKeyCode::Cyrillic_er,
        //x11::keysym::XK_Cyrillic_es => VirtualKeyCode::Cyrillic_es,
        //x11::keysym::XK_Cyrillic_te => VirtualKeyCode::Cyrillic_te,
        //x11::keysym::XK_Cyrillic_u => VirtualKeyCode::Cyrillic_u,
        //x11::keysym::XK_Cyrillic_zhe => VirtualKeyCode::Cyrillic_zhe,
        //x11::keysym::XK_Cyrillic_ve => VirtualKeyCode::Cyrillic_ve,
        //x11::keysym::XK_Cyrillic_softsign => VirtualKeyCode::Cyrillic_softsign,
        //x11::keysym::XK_Cyrillic_yeru => VirtualKeyCode::Cyrillic_yeru,
        //x11::keysym::XK_Cyrillic_ze => VirtualKeyCode::Cyrillic_ze,
        //x11::keysym::XK_Cyrillic_sha => VirtualKeyCode::Cyrillic_sha,
        //x11::keysym::XK_Cyrillic_e => VirtualKeyCode::Cyrillic_e,
        //x11::keysym::XK_Cyrillic_shcha => VirtualKeyCode::Cyrillic_shcha,
        //x11::keysym::XK_Cyrillic_che => VirtualKeyCode::Cyrillic_che,
        //x11::keysym::XK_Cyrillic_hardsign => VirtualKeyCode::Cyrillic_hardsign,
        //x11::keysym::XK_Cyrillic_YU => VirtualKeyCode::Cyrillic_yu,
        //x11::keysym::XK_Cyrillic_A => VirtualKeyCode::Cyrillic_a,
        //x11::keysym::XK_Cyrillic_BE => VirtualKeyCode::Cyrillic_be,
        //x11::keysym::XK_Cyrillic_TSE => VirtualKeyCode::Cyrillic_tse,
        //x11::keysym::XK_Cyrillic_DE => VirtualKeyCode::Cyrillic_de,
        //x11::keysym::XK_Cyrillic_IE => VirtualKeyCode::Cyrillic_ie,
        //x11::keysym::XK_Cyrillic_EF => VirtualKeyCode::Cyrillic_ef,
        //x11::keysym::XK_Cyrillic_GHE => VirtualKeyCode::Cyrillic_ghe,
        //x11::keysym::XK_Cyrillic_HA => VirtualKeyCode::Cyrillic_ha,
        //x11::keysym::XK_Cyrillic_I => VirtualKeyCode::Cyrillic_i,
        //x11::keysym::XK_Cyrillic_SHORTI => VirtualKeyCode::Cyrillic_shorti,
        //x11::keysym::XK_Cyrillic_KA => VirtualKeyCode::Cyrillic_ka,
        //x11::keysym::XK_Cyrillic_EL => VirtualKeyCode::Cyrillic_el,
        //x11::keysym::XK_Cyrillic_EM => VirtualKeyCode::Cyrillic_em,
        //x11::keysym::XK_Cyrillic_EN => VirtualKeyCode::Cyrillic_en,
        //x11::keysym::XK_Cyrillic_O => VirtualKeyCode::Cyrillic_o,
        //x11::keysym::XK_Cyrillic_PE => VirtualKeyCode::Cyrillic_pe,
        //x11::keysym::XK_Cyrillic_YA => VirtualKeyCode::Cyrillic_ya,
        //x11::keysym::XK_Cyrillic_ER => VirtualKeyCode::Cyrillic_er,
        //x11::keysym::XK_Cyrillic_ES => VirtualKeyCode::Cyrillic_es,
        //x11::keysym::XK_Cyrillic_TE => VirtualKeyCode::Cyrillic_te,
        //x11::keysym::XK_Cyrillic_U => VirtualKeyCode::Cyrillic_u,
        //x11::keysym::XK_Cyrillic_ZHE => VirtualKeyCode::Cyrillic_zhe,
        //x11::keysym::XK_Cyrillic_VE => VirtualKeyCode::Cyrillic_ve,
        //x11::keysym::XK_Cyrillic_SOFTSIGN => VirtualKeyCode::Cyrillic_softsign,
        //x11::keysym::XK_Cyrillic_YERU => VirtualKeyCode::Cyrillic_yeru,
        //x11::keysym::XK_Cyrillic_ZE => VirtualKeyCode::Cyrillic_ze,
        //x11::keysym::XK_Cyrillic_SHA => VirtualKeyCode::Cyrillic_sha,
        //x11::keysym::XK_Cyrillic_E => VirtualKeyCode::Cyrillic_e,
        //x11::keysym::XK_Cyrillic_SHCHA => VirtualKeyCode::Cyrillic_shcha,
        //x11::keysym::XK_Cyrillic_CHE => VirtualKeyCode::Cyrillic_che,
        //x11::keysym::XK_Cyrillic_HARDSIGN => VirtualKeyCode::Cyrillic_hardsign,
        //x11::keysym::XK_Greek_ALPHAaccent => VirtualKeyCode::Greek_alphaaccent,
        //x11::keysym::XK_Greek_EPSILONaccent => VirtualKeyCode::Greek_epsilonaccent,
        //x11::keysym::XK_Greek_ETAaccent => VirtualKeyCode::Greek_etaaccent,
        //x11::keysym::XK_Greek_IOTAaccent => VirtualKeyCode::Greek_iotaaccent,
        //x11::keysym::XK_Greek_IOTAdiaeresis => VirtualKeyCode::Greek_iotadiaeresis,
        //x11::keysym::XK_Greek_OMICRONaccent => VirtualKeyCode::Greek_omicronaccent,
        //x11::keysym::XK_Greek_UPSILONaccent => VirtualKeyCode::Greek_upsilonaccent,
        //x11::keysym::XK_Greek_UPSILONdieresis => VirtualKeyCode::Greek_upsilondieresis,
        //x11::keysym::XK_Greek_OMEGAaccent => VirtualKeyCode::Greek_omegaaccent,
        //x11::keysym::XK_Greek_accentdieresis => VirtualKeyCode::Greek_accentdieresis,
        //x11::keysym::XK_Greek_horizbar => VirtualKeyCode::Greek_horizbar,
        //x11::keysym::XK_Greek_alphaaccent => VirtualKeyCode::Greek_alphaaccent,
        //x11::keysym::XK_Greek_epsilonaccent => VirtualKeyCode::Greek_epsilonaccent,
        //x11::keysym::XK_Greek_etaaccent => VirtualKeyCode::Greek_etaaccent,
        //x11::keysym::XK_Greek_iotaaccent => VirtualKeyCode::Greek_iotaaccent,
        //x11::keysym::XK_Greek_iotadieresis => VirtualKeyCode::Greek_iotadieresis,
        //x11::keysym::XK_Greek_iotaaccentdieresis => VirtualKeyCode::Greek_iotaaccentdieresis,
        //x11::keysym::XK_Greek_omicronaccent => VirtualKeyCode::Greek_omicronaccent,
        //x11::keysym::XK_Greek_upsilonaccent => VirtualKeyCode::Greek_upsilonaccent,
        //x11::keysym::XK_Greek_upsilondieresis => VirtualKeyCode::Greek_upsilondieresis,
        //x11::keysym::XK_Greek_upsilonaccentdieresis => VirtualKeyCode::Greek_upsilonaccentdieresis,
        //x11::keysym::XK_Greek_omegaaccent => VirtualKeyCode::Greek_omegaaccent,
        //x11::keysym::XK_Greek_ALPHA => VirtualKeyCode::Greek_alpha,
        //x11::keysym::XK_Greek_BETA => VirtualKeyCode::Greek_beta,
        //x11::keysym::XK_Greek_GAMMA => VirtualKeyCode::Greek_gamma,
        //x11::keysym::XK_Greek_DELTA => VirtualKeyCode::Greek_delta,
        //x11::keysym::XK_Greek_EPSILON => VirtualKeyCode::Greek_epsilon,
        //x11::keysym::XK_Greek_ZETA => VirtualKeyCode::Greek_zeta,
        //x11::keysym::XK_Greek_ETA => VirtualKeyCode::Greek_eta,
        //x11::keysym::XK_Greek_THETA => VirtualKeyCode::Greek_theta,
        //x11::keysym::XK_Greek_IOTA => VirtualKeyCode::Greek_iota,
        //x11::keysym::XK_Greek_KAPPA => VirtualKeyCode::Greek_kappa,
        //x11::keysym::XK_Greek_LAMDA => VirtualKeyCode::Greek_lamda,
        //x11::keysym::XK_Greek_LAMBDA => VirtualKeyCode::Greek_lambda,
        //x11::keysym::XK_Greek_MU => VirtualKeyCode::Greek_mu,
        //x11::keysym::XK_Greek_NU => VirtualKeyCode::Greek_nu,
        //x11::keysym::XK_Greek_XI => VirtualKeyCode::Greek_xi,
        //x11::keysym::XK_Greek_OMICRON => VirtualKeyCode::Greek_omicron,
        //x11::keysym::XK_Greek_PI => VirtualKeyCode::Greek_pi,
        //x11::keysym::XK_Greek_RHO => VirtualKeyCode::Greek_rho,
        //x11::keysym::XK_Greek_SIGMA => VirtualKeyCode::Greek_sigma,
        //x11::keysym::XK_Greek_TAU => VirtualKeyCode::Greek_tau,
        //x11::keysym::XK_Greek_UPSILON => VirtualKeyCode::Greek_upsilon,
        //x11::keysym::XK_Greek_PHI => VirtualKeyCode::Greek_phi,
        //x11::keysym::XK_Greek_CHI => VirtualKeyCode::Greek_chi,
        //x11::keysym::XK_Greek_PSI => VirtualKeyCode::Greek_psi,
        //x11::keysym::XK_Greek_OMEGA => VirtualKeyCode::Greek_omega,
        //x11::keysym::XK_Greek_alpha => VirtualKeyCode::Greek_alpha,
        //x11::keysym::XK_Greek_beta => VirtualKeyCode::Greek_beta,
        //x11::keysym::XK_Greek_gamma => VirtualKeyCode::Greek_gamma,
        //x11::keysym::XK_Greek_delta => VirtualKeyCode::Greek_delta,
        //x11::keysym::XK_Greek_epsilon => VirtualKeyCode::Greek_epsilon,
        //x11::keysym::XK_Greek_zeta => VirtualKeyCode::Greek_zeta,
        //x11::keysym::XK_Greek_eta => VirtualKeyCode::Greek_eta,
        //x11::keysym::XK_Greek_theta => VirtualKeyCode::Greek_theta,
        //x11::keysym::XK_Greek_iota => VirtualKeyCode::Greek_iota,
        //x11::keysym::XK_Greek_kappa => VirtualKeyCode::Greek_kappa,
        //x11::keysym::XK_Greek_lamda => VirtualKeyCode::Greek_lamda,
        //x11::keysym::XK_Greek_lambda => VirtualKeyCode::Greek_lambda,
        //x11::keysym::XK_Greek_mu => VirtualKeyCode::Greek_mu,
        //x11::keysym::XK_Greek_nu => VirtualKeyCode::Greek_nu,
        //x11::keysym::XK_Greek_xi => VirtualKeyCode::Greek_xi,
        //x11::keysym::XK_Greek_omicron => VirtualKeyCode::Greek_omicron,
        //x11::keysym::XK_Greek_pi => VirtualKeyCode::Greek_pi,
        //x11::keysym::XK_Greek_rho => VirtualKeyCode::Greek_rho,
        //x11::keysym::XK_Greek_sigma => VirtualKeyCode::Greek_sigma,
        //x11::keysym::XK_Greek_finalsmallsigma => VirtualKeyCode::Greek_finalsmallsigma,
        //x11::keysym::XK_Greek_tau => VirtualKeyCode::Greek_tau,
        //x11::keysym::XK_Greek_upsilon => VirtualKeyCode::Greek_upsilon,
        //x11::keysym::XK_Greek_phi => VirtualKeyCode::Greek_phi,
        //x11::keysym::XK_Greek_chi => VirtualKeyCode::Greek_chi,
        //x11::keysym::XK_Greek_psi => VirtualKeyCode::Greek_psi,
        //x11::keysym::XK_Greek_omega => VirtualKeyCode::Greek_omega,
        //x11::keysym::XK_Greek_switch => VirtualKeyCode::Greek_switch,
        //x11::keysym::XK_leftradical => VirtualKeyCode::Leftradical,
        //x11::keysym::XK_topleftradical => VirtualKeyCode::Topleftradical,
        //x11::keysym::XK_horizconnector => VirtualKeyCode::Horizconnector,
        //x11::keysym::XK_topintegral => VirtualKeyCode::Topintegral,
        //x11::keysym::XK_botintegral => VirtualKeyCode::Botintegral,
        //x11::keysym::XK_vertconnector => VirtualKeyCode::Vertconnector,
        //x11::keysym::XK_topleftsqbracket => VirtualKeyCode::Topleftsqbracket,
        //x11::keysym::XK_botleftsqbracket => VirtualKeyCode::Botleftsqbracket,
        //x11::keysym::XK_toprightsqbracket => VirtualKeyCode::Toprightsqbracket,
        //x11::keysym::XK_botrightsqbracket => VirtualKeyCode::Botrightsqbracket,
        //x11::keysym::XK_topleftparens => VirtualKeyCode::Topleftparens,
        //x11::keysym::XK_botleftparens => VirtualKeyCode::Botleftparens,
        //x11::keysym::XK_toprightparens => VirtualKeyCode::Toprightparens,
        //x11::keysym::XK_botrightparens => VirtualKeyCode::Botrightparens,
        //x11::keysym::XK_leftmiddlecurlybrace => VirtualKeyCode::Leftmiddlecurlybrace,
        //x11::keysym::XK_rightmiddlecurlybrace => VirtualKeyCode::Rightmiddlecurlybrace,
        //x11::keysym::XK_topleftsummation => VirtualKeyCode::Topleftsummation,
        //x11::keysym::XK_botleftsummation => VirtualKeyCode::Botleftsummation,
        //x11::keysym::XK_topvertsummationconnector => VirtualKeyCode::Topvertsummationconnector,
        //x11::keysym::XK_botvertsummationconnector => VirtualKeyCode::Botvertsummationconnector,
        //x11::keysym::XK_toprightsummation => VirtualKeyCode::Toprightsummation,
        //x11::keysym::XK_botrightsummation => VirtualKeyCode::Botrightsummation,
        //x11::keysym::XK_rightmiddlesummation => VirtualKeyCode::Rightmiddlesummation,
        //x11::keysym::XK_lessthanequal => VirtualKeyCode::Lessthanequal,
        //x11::keysym::XK_notequal => VirtualKeyCode::Notequal,
        //x11::keysym::XK_greaterthanequal => VirtualKeyCode::Greaterthanequal,
        //x11::keysym::XK_integral => VirtualKeyCode::Integral,
        //x11::keysym::XK_therefore => VirtualKeyCode::Therefore,
        //x11::keysym::XK_variation => VirtualKeyCode::Variation,
        //x11::keysym::XK_infinity => VirtualKeyCode::Infinity,
        //x11::keysym::XK_nabla => VirtualKeyCode::Nabla,
        //x11::keysym::XK_approximate => VirtualKeyCode::Approximate,
        //x11::keysym::XK_similarequal => VirtualKeyCode::Similarequal,
        //x11::keysym::XK_ifonlyif => VirtualKeyCode::Ifonlyif,
        //x11::keysym::XK_implies => VirtualKeyCode::Implies,
        //x11::keysym::XK_identical => VirtualKeyCode::Identical,
        //x11::keysym::XK_radical => VirtualKeyCode::Radical,
        //x11::keysym::XK_includedin => VirtualKeyCode::Includedin,
        //x11::keysym::XK_includes => VirtualKeyCode::Includes,
        //x11::keysym::XK_intersection => VirtualKeyCode::Intersection,
        //x11::keysym::XK_union => VirtualKeyCode::Union,
        //x11::keysym::XK_logicaland => VirtualKeyCode::Logicaland,
        //x11::keysym::XK_logicalor => VirtualKeyCode::Logicalor,
        //x11::keysym::XK_partialderivative => VirtualKeyCode::Partialderivative,
        //x11::keysym::XK_function => VirtualKeyCode::Function,
        //x11::keysym::XK_leftarrow => VirtualKeyCode::Leftarrow,
        //x11::keysym::XK_uparrow => VirtualKeyCode::Uparrow,
        //x11::keysym::XK_rightarrow => VirtualKeyCode::Rightarrow,
        //x11::keysym::XK_downarrow => VirtualKeyCode::Downarrow,
        //x11::keysym::XK_blank => VirtualKeyCode::Blank,
        //x11::keysym::XK_soliddiamond => VirtualKeyCode::Soliddiamond,
        //x11::keysym::XK_checkerboard => VirtualKeyCode::Checkerboard,
        //x11::keysym::XK_ht => VirtualKeyCode::Ht,
        //x11::keysym::XK_ff => VirtualKeyCode::Ff,
        //x11::keysym::XK_cr => VirtualKeyCode::Cr,
        //x11::keysym::XK_lf => VirtualKeyCode::Lf,
        //x11::keysym::XK_nl => VirtualKeyCode::Nl,
        //x11::keysym::XK_vt => VirtualKeyCode::Vt,
        //x11::keysym::XK_lowrightcorner => VirtualKeyCode::Lowrightcorner,
        //x11::keysym::XK_uprightcorner => VirtualKeyCode::Uprightcorner,
        //x11::keysym::XK_upleftcorner => VirtualKeyCode::Upleftcorner,
        //x11::keysym::XK_lowleftcorner => VirtualKeyCode::Lowleftcorner,
        //x11::keysym::XK_crossinglines => VirtualKeyCode::Crossinglines,
        //x11::keysym::XK_horizlinescan1 => VirtualKeyCode::Horizlinescan1,
        //x11::keysym::XK_horizlinescan3 => VirtualKeyCode::Horizlinescan3,
        //x11::keysym::XK_horizlinescan5 => VirtualKeyCode::Horizlinescan5,
        //x11::keysym::XK_horizlinescan7 => VirtualKeyCode::Horizlinescan7,
        //x11::keysym::XK_horizlinescan9 => VirtualKeyCode::Horizlinescan9,
        //x11::keysym::XK_leftt => VirtualKeyCode::Leftt,
        //x11::keysym::XK_rightt => VirtualKeyCode::Rightt,
        //x11::keysym::XK_bott => VirtualKeyCode::Bott,
        //x11::keysym::XK_topt => VirtualKeyCode::Topt,
        //x11::keysym::XK_vertbar => VirtualKeyCode::Vertbar,
        //x11::keysym::XK_emspace => VirtualKeyCode::Emspace,
        //x11::keysym::XK_enspace => VirtualKeyCode::Enspace,
        //x11::keysym::XK_em3space => VirtualKeyCode::Em3space,
        //x11::keysym::XK_em4space => VirtualKeyCode::Em4space,
        //x11::keysym::XK_digitspace => VirtualKeyCode::Digitspace,
        //x11::keysym::XK_punctspace => VirtualKeyCode::Punctspace,
        //x11::keysym::XK_thinspace => VirtualKeyCode::Thinspace,
        //x11::keysym::XK_hairspace => VirtualKeyCode::Hairspace,
        //x11::keysym::XK_emdash => VirtualKeyCode::Emdash,
        //x11::keysym::XK_endash => VirtualKeyCode::Endash,
        //x11::keysym::XK_signifblank => VirtualKeyCode::Signifblank,
        //x11::keysym::XK_ellipsis => VirtualKeyCode::Ellipsis,
        //x11::keysym::XK_doubbaselinedot => VirtualKeyCode::Doubbaselinedot,
        //x11::keysym::XK_onethird => VirtualKeyCode::Onethird,
        //x11::keysym::XK_twothirds => VirtualKeyCode::Twothirds,
        //x11::keysym::XK_onefifth => VirtualKeyCode::Onefifth,
        //x11::keysym::XK_twofifths => VirtualKeyCode::Twofifths,
        //x11::keysym::XK_threefifths => VirtualKeyCode::Threefifths,
        //x11::keysym::XK_fourfifths => VirtualKeyCode::Fourfifths,
        //x11::keysym::XK_onesixth => VirtualKeyCode::Onesixth,
        //x11::keysym::XK_fivesixths => VirtualKeyCode::Fivesixths,
        //x11::keysym::XK_careof => VirtualKeyCode::Careof,
        //x11::keysym::XK_figdash => VirtualKeyCode::Figdash,
        //x11::keysym::XK_leftanglebracket => VirtualKeyCode::Leftanglebracket,
        //x11::keysym::XK_decimalpoint => VirtualKeyCode::Decimalpoint,
        //x11::keysym::XK_rightanglebracket => VirtualKeyCode::Rightanglebracket,
        //x11::keysym::XK_marker => VirtualKeyCode::Marker,
        //x11::keysym::XK_oneeighth => VirtualKeyCode::Oneeighth,
        //x11::keysym::XK_threeeighths => VirtualKeyCode::Threeeighths,
        //x11::keysym::XK_fiveeighths => VirtualKeyCode::Fiveeighths,
        //x11::keysym::XK_seveneighths => VirtualKeyCode::Seveneighths,
        //x11::keysym::XK_trademark => VirtualKeyCode::Trademark,
        //x11::keysym::XK_signaturemark => VirtualKeyCode::Signaturemark,
        //x11::keysym::XK_trademarkincircle => VirtualKeyCode::Trademarkincircle,
        //x11::keysym::XK_leftopentriangle => VirtualKeyCode::Leftopentriangle,
        //x11::keysym::XK_rightopentriangle => VirtualKeyCode::Rightopentriangle,
        //x11::keysym::XK_emopencircle => VirtualKeyCode::Emopencircle,
        //x11::keysym::XK_emopenrectangle => VirtualKeyCode::Emopenrectangle,
        //x11::keysym::XK_leftsinglequotemark => VirtualKeyCode::Leftsinglequotemark,
        //x11::keysym::XK_rightsinglequotemark => VirtualKeyCode::Rightsinglequotemark,
        //x11::keysym::XK_leftdoublequotemark => VirtualKeyCode::Leftdoublequotemark,
        //x11::keysym::XK_rightdoublequotemark => VirtualKeyCode::Rightdoublequotemark,
        //x11::keysym::XK_prescription => VirtualKeyCode::Prescription,
        //x11::keysym::XK_minutes => VirtualKeyCode::Minutes,
        //x11::keysym::XK_seconds => VirtualKeyCode::Seconds,
        //x11::keysym::XK_latincross => VirtualKeyCode::Latincross,
        //x11::keysym::XK_hexagram => VirtualKeyCode::Hexagram,
        //x11::keysym::XK_filledrectbullet => VirtualKeyCode::Filledrectbullet,
        //x11::keysym::XK_filledlefttribullet => VirtualKeyCode::Filledlefttribullet,
        //x11::keysym::XK_filledrighttribullet => VirtualKeyCode::Filledrighttribullet,
        //x11::keysym::XK_emfilledcircle => VirtualKeyCode::Emfilledcircle,
        //x11::keysym::XK_emfilledrect => VirtualKeyCode::Emfilledrect,
        //x11::keysym::XK_enopencircbullet => VirtualKeyCode::Enopencircbullet,
        //x11::keysym::XK_enopensquarebullet => VirtualKeyCode::Enopensquarebullet,
        //x11::keysym::XK_openrectbullet => VirtualKeyCode::Openrectbullet,
        //x11::keysym::XK_opentribulletup => VirtualKeyCode::Opentribulletup,
        //x11::keysym::XK_opentribulletdown => VirtualKeyCode::Opentribulletdown,
        //x11::keysym::XK_openstar => VirtualKeyCode::Openstar,
        //x11::keysym::XK_enfilledcircbullet => VirtualKeyCode::Enfilledcircbullet,
        //x11::keysym::XK_enfilledsqbullet => VirtualKeyCode::Enfilledsqbullet,
        //x11::keysym::XK_filledtribulletup => VirtualKeyCode::Filledtribulletup,
        //x11::keysym::XK_filledtribulletdown => VirtualKeyCode::Filledtribulletdown,
        //x11::keysym::XK_leftpointer => VirtualKeyCode::Leftpointer,
        //x11::keysym::XK_rightpointer => VirtualKeyCode::Rightpointer,
        //x11::keysym::XK_club => VirtualKeyCode::Club,
        //x11::keysym::XK_diamond => VirtualKeyCode::Diamond,
        //x11::keysym::XK_heart => VirtualKeyCode::Heart,
        //x11::keysym::XK_maltesecross => VirtualKeyCode::Maltesecross,
        //x11::keysym::XK_dagger => VirtualKeyCode::Dagger,
        //x11::keysym::XK_doubledagger => VirtualKeyCode::Doubledagger,
        //x11::keysym::XK_checkmark => VirtualKeyCode::Checkmark,
        //x11::keysym::XK_ballotcross => VirtualKeyCode::Ballotcross,
        //x11::keysym::XK_musicalsharp => VirtualKeyCode::Musicalsharp,
        //x11::keysym::XK_musicalflat => VirtualKeyCode::Musicalflat,
        //x11::keysym::XK_malesymbol => VirtualKeyCode::Malesymbol,
        //x11::keysym::XK_femalesymbol => VirtualKeyCode::Femalesymbol,
        //x11::keysym::XK_telephone => VirtualKeyCode::Telephone,
        //x11::keysym::XK_telephonerecorder => VirtualKeyCode::Telephonerecorder,
        //x11::keysym::XK_phonographcopyright => VirtualKeyCode::Phonographcopyright,
        //x11::keysym::XK_caret => VirtualKeyCode::Caret,
        //x11::keysym::XK_singlelowquotemark => VirtualKeyCode::Singlelowquotemark,
        //x11::keysym::XK_doublelowquotemark => VirtualKeyCode::Doublelowquotemark,
        //x11::keysym::XK_cursor => VirtualKeyCode::Cursor,
        //x11::keysym::XK_leftcaret => VirtualKeyCode::Leftcaret,
        //x11::keysym::XK_rightcaret => VirtualKeyCode::Rightcaret,
        //x11::keysym::XK_downcaret => VirtualKeyCode::Downcaret,
        //x11::keysym::XK_upcaret => VirtualKeyCode::Upcaret,
        //x11::keysym::XK_overbar => VirtualKeyCode::Overbar,
        //x11::keysym::XK_downtack => VirtualKeyCode::Downtack,
        //x11::keysym::XK_upshoe => VirtualKeyCode::Upshoe,
        //x11::keysym::XK_downstile => VirtualKeyCode::Downstile,
        //x11::keysym::XK_underbar => VirtualKeyCode::Underbar,
        //x11::keysym::XK_jot => VirtualKeyCode::Jot,
        //x11::keysym::XK_quad => VirtualKeyCode::Quad,
        //x11::keysym::XK_uptack => VirtualKeyCode::Uptack,
        //x11::keysym::XK_circle => VirtualKeyCode::Circle,
        //x11::keysym::XK_upstile => VirtualKeyCode::Upstile,
        //x11::keysym::XK_downshoe => VirtualKeyCode::Downshoe,
        //x11::keysym::XK_rightshoe => VirtualKeyCode::Rightshoe,
        //x11::keysym::XK_leftshoe => VirtualKeyCode::Leftshoe,
        //x11::keysym::XK_lefttack => VirtualKeyCode::Lefttack,
        //x11::keysym::XK_righttack => VirtualKeyCode::Righttack,
        //x11::keysym::XK_hebrew_doublelowline => VirtualKeyCode::Hebrew_doublelowline,
        //x11::keysym::XK_hebrew_aleph => VirtualKeyCode::Hebrew_aleph,
        //x11::keysym::XK_hebrew_bet => VirtualKeyCode::Hebrew_bet,
        //x11::keysym::XK_hebrew_beth => VirtualKeyCode::Hebrew_beth,
        //x11::keysym::XK_hebrew_gimel => VirtualKeyCode::Hebrew_gimel,
        //x11::keysym::XK_hebrew_gimmel => VirtualKeyCode::Hebrew_gimmel,
        //x11::keysym::XK_hebrew_dalet => VirtualKeyCode::Hebrew_dalet,
        //x11::keysym::XK_hebrew_daleth => VirtualKeyCode::Hebrew_daleth,
        //x11::keysym::XK_hebrew_he => VirtualKeyCode::Hebrew_he,
        //x11::keysym::XK_hebrew_waw => VirtualKeyCode::Hebrew_waw,
        //x11::keysym::XK_hebrew_zain => VirtualKeyCode::Hebrew_zain,
        //x11::keysym::XK_hebrew_zayin => VirtualKeyCode::Hebrew_zayin,
        //x11::keysym::XK_hebrew_chet => VirtualKeyCode::Hebrew_chet,
        //x11::keysym::XK_hebrew_het => VirtualKeyCode::Hebrew_het,
        //x11::keysym::XK_hebrew_tet => VirtualKeyCode::Hebrew_tet,
        //x11::keysym::XK_hebrew_teth => VirtualKeyCode::Hebrew_teth,
        //x11::keysym::XK_hebrew_yod => VirtualKeyCode::Hebrew_yod,
        //x11::keysym::XK_hebrew_finalkaph => VirtualKeyCode::Hebrew_finalkaph,
        //x11::keysym::XK_hebrew_kaph => VirtualKeyCode::Hebrew_kaph,
        //x11::keysym::XK_hebrew_lamed => VirtualKeyCode::Hebrew_lamed,
        //x11::keysym::XK_hebrew_finalmem => VirtualKeyCode::Hebrew_finalmem,
        //x11::keysym::XK_hebrew_mem => VirtualKeyCode::Hebrew_mem,
        //x11::keysym::XK_hebrew_finalnun => VirtualKeyCode::Hebrew_finalnun,
        //x11::keysym::XK_hebrew_nun => VirtualKeyCode::Hebrew_nun,
        //x11::keysym::XK_hebrew_samech => VirtualKeyCode::Hebrew_samech,
        //x11::keysym::XK_hebrew_samekh => VirtualKeyCode::Hebrew_samekh,
        //x11::keysym::XK_hebrew_ayin => VirtualKeyCode::Hebrew_ayin,
        //x11::keysym::XK_hebrew_finalpe => VirtualKeyCode::Hebrew_finalpe,
        //x11::keysym::XK_hebrew_pe => VirtualKeyCode::Hebrew_pe,
        //x11::keysym::XK_hebrew_finalzade => VirtualKeyCode::Hebrew_finalzade,
        //x11::keysym::XK_hebrew_finalzadi => VirtualKeyCode::Hebrew_finalzadi,
        //x11::keysym::XK_hebrew_zade => VirtualKeyCode::Hebrew_zade,
        //x11::keysym::XK_hebrew_zadi => VirtualKeyCode::Hebrew_zadi,
        //x11::keysym::XK_hebrew_qoph => VirtualKeyCode::Hebrew_qoph,
        //x11::keysym::XK_hebrew_kuf => VirtualKeyCode::Hebrew_kuf,
        //x11::keysym::XK_hebrew_resh => VirtualKeyCode::Hebrew_resh,
        //x11::keysym::XK_hebrew_shin => VirtualKeyCode::Hebrew_shin,
        //x11::keysym::XK_hebrew_taw => VirtualKeyCode::Hebrew_taw,
        //x11::keysym::XK_hebrew_taf => VirtualKeyCode::Hebrew_taf,
        //x11::keysym::XK_Hebrew_switch => VirtualKeyCode::Hebrew_switch,
        x11::keysym::XF86XK_Back => VirtualKeyCode::NavigateBackward,
        x11::keysym::XF86XK_Forward => VirtualKeyCode::NavigateForward,
        x11::keysym::XF86XK_Copy => VirtualKeyCode::Copy,
        x11::keysym::XF86XK_Paste => VirtualKeyCode::Paste,
        x11::keysym::XF86XK_Cut => VirtualKeyCode::Cut,
        _ => return None,
    })
}

pub fn virtual_keycode_to_keysym(keycode: VirtualKeyCode) -> Option<u32> {
    Some(match keycode {
        VirtualKeyCode::Back => x11::keysym::XK_BackSpace,
        VirtualKeyCode::Tab => x11::keysym::XK_Tab,
        //VirtualKeyCode::Linefeed => x11::keysym::XK_Linefeed,
        //VirtualKeyCode::Clear => x11::keysym::XK_Clear,
        VirtualKeyCode::Return => x11::keysym::XK_Return,
        //VirtualKeyCode::Pause => x11::keysym::XK_Pause,
        //VirtualKeyCode::Scroll_lock => x11::keysym::XK_Scroll_Lock,
        //VirtualKeyCode::Sys_req => x11::keysym::XK_Sys_Req,
        VirtualKeyCode::Escape => x11::keysym::XK_Escape,
        VirtualKeyCode::Delete => x11::keysym::XK_Delete,
        VirtualKeyCode::Compose => x11::keysym::XK_Multi_key,
        //VirtualKeyCode::Kanji => x11::keysym::XK_Kanji,
        //VirtualKeyCode::Muhenkan => x11::keysym::XK_Muhenkan,
        //VirtualKeyCode::Henkan_mode => x11::keysym::XK_Henkan_Mode,
        //VirtualKeyCode::Henkan => x11::keysym::XK_Henkan,
        //VirtualKeyCode::Romaji => x11::keysym::XK_Romaji,
        //VirtualKeyCode::Hiragana => x11::keysym::XK_Hiragana,
        //VirtualKeyCode::Katakana => x11::keysym::XK_Katakana,
        //VirtualKeyCode::Hiragana_katakana => x11::keysym::XK_Hiragana_Katakana,
        //VirtualKeyCode::Zenkaku => x11::keysym::XK_Zenkaku,
        //VirtualKeyCode::Hankaku => x11::keysym::XK_Hankaku,
        //VirtualKeyCode::Zenkaku_hankaku => x11::keysym::XK_Zenkaku_Hankaku,
        //VirtualKeyCode::Touroku => x11::keysym::XK_Touroku,
        //VirtualKeyCode::Massyo => x11::keysym::XK_Massyo,
        //VirtualKeyCode::Kana_lock => x11::keysym::XK_Kana_Lock,
        //VirtualKeyCode::Kana_shift => x11::keysym::XK_Kana_Shift,
        //VirtualKeyCode::Eisu_shift => x11::keysym::XK_Eisu_Shift,
        //VirtualKeyCode::Eisu_toggle => x11::keysym::XK_Eisu_toggle,
        VirtualKeyCode::Home => x11::keysym::XK_Home,
        VirtualKeyCode::Left => x11::keysym::XK_Left,
        VirtualKeyCode::Up => x11::keysym::XK_Up,
        VirtualKeyCode::Right => x11::keysym::XK_Right,
        VirtualKeyCode::Down => x11::keysym::XK_Down,
        //VirtualKeyCode::Prior => x11::keysym::XK_Prior,
        VirtualKeyCode::PageUp => x11::keysym::XK_Page_Up,
        //VirtualKeyCode::Next => x11::keysym::XK_Next,
        VirtualKeyCode::PageDown => x11::keysym::XK_Page_Down,
        VirtualKeyCode::End => x11::keysym::XK_End,
        //VirtualKeyCode::Begin => x11::keysym::XK_Begin,
        VirtualKeyCode::LWin => x11::keysym::XK_Win_L,
        VirtualKeyCode::RWin => x11::keysym::XK_Win_R,
        //VirtualKeyCode::App => x11::keysym::XK_App,
        //VirtualKeyCode::Select => x11::keysym::XK_Select,
        VirtualKeyCode::Print => x11::keysym::XK_Print,
        //VirtualKeyCode::Execute => x11::keysym::XK_Execute,
        VirtualKeyCode::Insert => x11::keysym::XK_Insert,
        //VirtualKeyCode::Undo => x11::keysym::XK_Undo,
        //VirtualKeyCode::Redo => x11::keysym::XK_Redo,
        //VirtualKeyCode::Menu => x11::keysym::XK_Menu,
        //VirtualKeyCode::Find => x11::keysym::XK_Find,
        //VirtualKeyCode::Cancel => x11::keysym::XK_Cancel,
        //VirtualKeyCode::Help => x11::keysym::XK_Help,
        //VirtualKeyCode::Break => x11::keysym::XK_Break,
        //VirtualKeyCode::Mode_switch => x11::keysym::XK_Mode_switch,
        //VirtualKeyCode::Script_switch => x11::keysym::XK_script_switch,
        //VirtualKeyCode::Num_lock => x11::keysym::XK_Num_Lock,
        //VirtualKeyCode::Kp_space => x11::keysym::XK_KP_Space,
        //VirtualKeyCode::Kp_tab => x11::keysym::XK_KP_Tab,
        //VirtualKeyCode::Kp_enter => x11::keysym::XK_KP_Enter,
        //VirtualKeyCode::Kp_f1 => x11::keysym::XK_KP_F1,
        //VirtualKeyCode::Kp_f2 => x11::keysym::XK_KP_F2,
        //VirtualKeyCode::Kp_f3 => x11::keysym::XK_KP_F3,
        //VirtualKeyCode::Kp_f4 => x11::keysym::XK_KP_F4,
        VirtualKeyCode::Home => x11::keysym::XK_KP_Home,
        VirtualKeyCode::Left => x11::keysym::XK_KP_Left,
        VirtualKeyCode::Up => x11::keysym::XK_KP_Up,
        VirtualKeyCode::Right => x11::keysym::XK_KP_Right,
        VirtualKeyCode::Down => x11::keysym::XK_KP_Down,
        //VirtualKeyCode::Kp_prior => x11::keysym::XK_KP_Prior,
        VirtualKeyCode::PageUp => x11::keysym::XK_KP_Page_Up,
        //VirtualKeyCode::Kp_next => x11::keysym::XK_KP_Next,
        VirtualKeyCode::PageDown => x11::keysym::XK_KP_Page_Down,
        VirtualKeyCode::End => x11::keysym::XK_KP_End,
        //VirtualKeyCode::Kp_begin => x11::keysym::XK_KP_Begin,
        VirtualKeyCode::Insert => x11::keysym::XK_KP_Insert,
        VirtualKeyCode::Delete => x11::keysym::XK_KP_Delete,
        VirtualKeyCode::NumpadEquals => x11::keysym::XK_KP_Equal,
        VirtualKeyCode::NumpadMultiply => x11::keysym::XK_KP_Multiply,
        VirtualKeyCode::NumpadAdd => x11::keysym::XK_KP_Add,
        //VirtualKeyCode::Kp_separator => x11::keysym::XK_KP_Separator,
        VirtualKeyCode::NumpadSubtract => x11::keysym::XK_KP_Subtract,
        VirtualKeyCode::NumpadDecimal => x11::keysym::XK_KP_Decimal,
        VirtualKeyCode::NumpadDivide => x11::keysym::XK_KP_Divide,
        VirtualKeyCode::Numpad0 => x11::keysym::XK_KP_0,
        VirtualKeyCode::Numpad1 => x11::keysym::XK_KP_1,
        VirtualKeyCode::Numpad2 => x11::keysym::XK_KP_2,
        VirtualKeyCode::Numpad3 => x11::keysym::XK_KP_3,
        VirtualKeyCode::Numpad4 => x11::keysym::XK_KP_4,
        VirtualKeyCode::Numpad5 => x11::keysym::XK_KP_5,
        VirtualKeyCode::Numpad6 => x11::keysym::XK_KP_6,
        VirtualKeyCode::Numpad7 => x11::keysym::XK_KP_7,
        VirtualKeyCode::Numpad8 => x11::keysym::XK_KP_8,
        VirtualKeyCode::Numpad9 => x11::keysym::XK_KP_9,
        VirtualKeyCode::F1 => x11::keysym::XK_F1,
        VirtualKeyCode::F2 => x11::keysym::XK_F2,
        VirtualKeyCode::F3 => x11::keysym::XK_F3,
        VirtualKeyCode::F4 => x11::keysym::XK_F4,
        VirtualKeyCode::F5 => x11::keysym::XK_F5,
        VirtualKeyCode::F6 => x11::keysym::XK_F6,
        VirtualKeyCode::F7 => x11::keysym::XK_F7,
        VirtualKeyCode::F8 => x11::keysym::XK_F8,
        VirtualKeyCode::F9 => x11::keysym::XK_F9,
        VirtualKeyCode::F10 => x11::keysym::XK_F10,
        VirtualKeyCode::F11 => x11::keysym::XK_F11,
        //VirtualKeyCode::L1 => x11::keysym::XK_L1,
        VirtualKeyCode::F12 => x11::keysym::XK_F12,
        //VirtualKeyCode::L2 => x11::keysym::XK_L2,
        VirtualKeyCode::F13 => x11::keysym::XK_F13,
        //VirtualKeyCode::L3 => x11::keysym::XK_L3,
        VirtualKeyCode::F14 => x11::keysym::XK_F14,
        //VirtualKeyCode::L4 => x11::keysym::XK_L4,
        VirtualKeyCode::F15 => x11::keysym::XK_F15,
        //VirtualKeyCode::L5 => x11::keysym::XK_L5,
        VirtualKeyCode::F16 => x11::keysym::XK_F16,
        //VirtualKeyCode::L6 => x11::keysym::XK_L6,
        VirtualKeyCode::F17 => x11::keysym::XK_F17,
        //VirtualKeyCode::L7 => x11::keysym::XK_L7,
        VirtualKeyCode::F18 => x11::keysym::XK_F18,
        //VirtualKeyCode::L8 => x11::keysym::XK_L8,
        VirtualKeyCode::F19 => x11::keysym::XK_F19,
        //VirtualKeyCode::L9 => x11::keysym::XK_L9,
        VirtualKeyCode::F20 => x11::keysym::XK_F20,
        //VirtualKeyCode::L10 => x11::keysym::XK_L10,
        VirtualKeyCode::F21 => x11::keysym::XK_F21,
        //VirtualKeyCode::R1 => x11::keysym::XK_R1,
        VirtualKeyCode::F22 => x11::keysym::XK_F22,
        //VirtualKeyCode::R2 => x11::keysym::XK_R2,
        VirtualKeyCode::F23 => x11::keysym::XK_F23,
        //VirtualKeyCode::R3 => x11::keysym::XK_R3,
        VirtualKeyCode::F24 => x11::keysym::XK_F24,
        //VirtualKeyCode::R4 => x11::keysym::XK_R4,
        //VirtualKeyCode::F25 => x11::keysym::XK_F25,
        //VirtualKeyCode::R5 => x11::keysym::XK_R5,
        //VirtualKeyCode::F26 => x11::keysym::XK_F26,
        //VirtualKeyCode::R6 => x11::keysym::XK_R6,
        //VirtualKeyCode::F27 => x11::keysym::XK_F27,
        //VirtualKeyCode::R7 => x11::keysym::XK_R7,
        //VirtualKeyCode::F28 => x11::keysym::XK_F28,
        //VirtualKeyCode::R8 => x11::keysym::XK_R8,
        //VirtualKeyCode::F29 => x11::keysym::XK_F29,
        //VirtualKeyCode::R9 => x11::keysym::XK_R9,
        //VirtualKeyCode::F30 => x11::keysym::XK_F30,
        //VirtualKeyCode::R10 => x11::keysym::XK_R10,
        //VirtualKeyCode::F31 => x11::keysym::XK_F31,
        //VirtualKeyCode::R11 => x11::keysym::XK_R11,
        //VirtualKeyCode::F32 => x11::keysym::XK_F32,
        //VirtualKeyCode::R12 => x11::keysym::XK_R12,
        //VirtualKeyCode::F33 => x11::keysym::XK_F33,
        //VirtualKeyCode::R13 => x11::keysym::XK_R13,
        //VirtualKeyCode::F34 => x11::keysym::XK_F34,
        //VirtualKeyCode::R14 => x11::keysym::XK_R14,
        //VirtualKeyCode::F35 => x11::keysym::XK_F35,
        //VirtualKeyCode::R15 => x11::keysym::XK_R15,
        VirtualKeyCode::LShift => x11::keysym::XK_Shift_L,
        VirtualKeyCode::RShift => x11::keysym::XK_Shift_R,
        VirtualKeyCode::LControl => x11::keysym::XK_Control_L,
        VirtualKeyCode::RControl => x11::keysym::XK_Control_R,
        //VirtualKeyCode::Caps_lock => x11::keysym::XK_Caps_Lock,
        //VirtualKeyCode::Shift_lock => x11::keysym::XK_Shift_Lock,
        //VirtualKeyCode::Meta_l => x11::keysym::XK_Meta_L,
        //VirtualKeyCode::Meta_r => x11::keysym::XK_Meta_R,
        VirtualKeyCode::LAlt => x11::keysym::XK_Alt_L,
        VirtualKeyCode::RAlt => x11::keysym::XK_Alt_R,
        //VirtualKeyCode::Super_l => x11::keysym::XK_Super_L,
        //VirtualKeyCode::Super_r => x11::keysym::XK_Super_R,
        //VirtualKeyCode::Hyper_l => x11::keysym::XK_Hyper_L,
        //VirtualKeyCode::Hyper_r => x11::keysym::XK_Hyper_R,
        VirtualKeyCode::Tab => x11::keysym::XK_ISO_Left_Tab,
        VirtualKeyCode::Space => x11::keysym::XK_space,
        //VirtualKeyCode::Exclam => x11::keysym::XK_exclam,
        //VirtualKeyCode::Quotedbl => x11::keysym::XK_quotedbl,
        //VirtualKeyCode::Numbersign => x11::keysym::XK_numbersign,
        //VirtualKeyCode::Dollar => x11::keysym::XK_dollar,
        //VirtualKeyCode::Percent => x11::keysym::XK_percent,
        //VirtualKeyCode::Ampersand => x11::keysym::XK_ampersand,
        VirtualKeyCode::Apostrophe => x11::keysym::XK_apostrophe,
        //VirtualKeyCode::Quoteright => x11::keysym::XK_quoteright,
        //VirtualKeyCode::Parenleft => x11::keysym::XK_parenleft,
        //VirtualKeyCode::Parenright => x11::keysym::XK_parenright,
        VirtualKeyCode::Asterisk => x11::keysym::XK_asterisk,
        VirtualKeyCode::Plus => x11::keysym::XK_plus,
        VirtualKeyCode::Comma => x11::keysym::XK_comma,
        VirtualKeyCode::Minus => x11::keysym::XK_minus,
        VirtualKeyCode::Period => x11::keysym::XK_period,
        VirtualKeyCode::Slash => x11::keysym::XK_slash,
        VirtualKeyCode::Zero => x11::keysym::XK_0,
        VirtualKeyCode::One => x11::keysym::XK_1,
        VirtualKeyCode::Two => x11::keysym::XK_2,
        VirtualKeyCode::Three => x11::keysym::XK_3,
        VirtualKeyCode::Four => x11::keysym::XK_4,
        VirtualKeyCode::Five => x11::keysym::XK_5,
        VirtualKeyCode::Six => x11::keysym::XK_6,
        VirtualKeyCode::Seven => x11::keysym::XK_7,
        VirtualKeyCode::Eight => x11::keysym::XK_8,
        VirtualKeyCode::Nine => x11::keysym::XK_9,
        VirtualKeyCode::Colon => x11::keysym::XK_colon,
        VirtualKeyCode::Semicolon => x11::keysym::XK_semicolon,
        //VirtualKeyCode::Less => x11::keysym::XK_less,
        VirtualKeyCode::Equals => x11::keysym::XK_equal,
        //VirtualKeyCode::Greater => x11::keysym::XK_greater,
        //VirtualKeyCode::Question => x11::keysym::XK_question,
        VirtualKeyCode::At => x11::keysym::XK_at,
        VirtualKeyCode::A => x11::keysym::XK_A,
        VirtualKeyCode::B => x11::keysym::XK_B,
        VirtualKeyCode::C => x11::keysym::XK_C,
        VirtualKeyCode::D => x11::keysym::XK_D,
        VirtualKeyCode::E => x11::keysym::XK_E,
        VirtualKeyCode::F => x11::keysym::XK_F,
        VirtualKeyCode::G => x11::keysym::XK_G,
        VirtualKeyCode::H => x11::keysym::XK_H,
        VirtualKeyCode::I => x11::keysym::XK_I,
        VirtualKeyCode::J => x11::keysym::XK_J,
        VirtualKeyCode::K => x11::keysym::XK_K,
        VirtualKeyCode::L => x11::keysym::XK_L,
        VirtualKeyCode::M => x11::keysym::XK_M,
        VirtualKeyCode::N => x11::keysym::XK_N,
        VirtualKeyCode::O => x11::keysym::XK_O,
        VirtualKeyCode::P => x11::keysym::XK_P,
        VirtualKeyCode::Q => x11::keysym::XK_Q,
        VirtualKeyCode::R => x11::keysym::XK_R,
        VirtualKeyCode::S => x11::keysym::XK_S,
        VirtualKeyCode::T => x11::keysym::XK_T,
        VirtualKeyCode::U => x11::keysym::XK_U,
        VirtualKeyCode::V => x11::keysym::XK_V,
        VirtualKeyCode::W => x11::keysym::XK_W,
        VirtualKeyCode::X => x11::keysym::XK_X,
        VirtualKeyCode::Y => x11::keysym::XK_Y,
        VirtualKeyCode::Z => x11::keysym::XK_Z,
        VirtualKeyCode::LBracket => x11::keysym::XK_bracketleft,
        VirtualKeyCode::Backslash => x11::keysym::XK_backslash,
        VirtualKeyCode::RBracket => x11::keysym::XK_bracketright,
        //VirtualKeyCode::Asciicircum => x11::keysym::XK_asciicircum,
        //VirtualKeyCode::Underscore => x11::keysym::XK_underscore,
        VirtualKeyCode::Grave => x11::keysym::XK_grave,
        //VirtualKeyCode::Quoteleft => x11::keysym::XK_quoteleft,
        VirtualKeyCode::A => x11::keysym::XK_a,
        VirtualKeyCode::B => x11::keysym::XK_b,
        VirtualKeyCode::C => x11::keysym::XK_c,
        VirtualKeyCode::D => x11::keysym::XK_d,
        VirtualKeyCode::E => x11::keysym::XK_e,
        VirtualKeyCode::F => x11::keysym::XK_f,
        VirtualKeyCode::G => x11::keysym::XK_g,
        VirtualKeyCode::H => x11::keysym::XK_h,
        VirtualKeyCode::I => x11::keysym::XK_i,
        VirtualKeyCode::J => x11::keysym::XK_j,
        VirtualKeyCode::K => x11::keysym::XK_k,
        VirtualKeyCode::L => x11::keysym::XK_l,
        VirtualKeyCode::M => x11::keysym::XK_m,
        VirtualKeyCode::N => x11::keysym::XK_n,
        VirtualKeyCode::O => x11::keysym::XK_o,
        VirtualKeyCode::P => x11::keysym::XK_p,
        VirtualKeyCode::Q => x11::keysym::XK_q,
        VirtualKeyCode::R => x11::keysym::XK_r,
        VirtualKeyCode::S => x11::keysym::XK_s,
        VirtualKeyCode::T => x11::keysym::XK_t,
        VirtualKeyCode::U => x11::keysym::XK_u,
        VirtualKeyCode::V => x11::keysym::XK_v,
        VirtualKeyCode::W => x11::keysym::XK_w,
        VirtualKeyCode::X => x11::keysym::XK_x,
        VirtualKeyCode::Y => x11::keysym::XK_y,
        VirtualKeyCode::Z => x11::keysym::XK_z,
        //VirtualKeyCode::Braceleft => x11::keysym::XK_braceleft,
        //VirtualKeyCode::Bar => x11::keysym::XK_bar,
        //VirtualKeyCode::Braceright => x11::keysym::XK_braceright,
        //VirtualKeyCode::Asciitilde => x11::keysym::XK_asciitilde,
        //VirtualKeyCode::Nobreakspace => x11::keysym::XK_nobreakspace,
        //VirtualKeyCode::Exclamdown => x11::keysym::XK_exclamdown,
        //VirtualKeyCode::Cent => x11::keysym::XK_cent,
        //VirtualKeyCode::Sterling => x11::keysym::XK_sterling,
        //VirtualKeyCode::Currency => x11::keysym::XK_currency,
        //VirtualKeyCode::Yen => x11::keysym::XK_yen,
        //VirtualKeyCode::Brokenbar => x11::keysym::XK_brokenbar,
        //VirtualKeyCode::Section => x11::keysym::XK_section,
        //VirtualKeyCode::Diaeresis => x11::keysym::XK_diaeresis,
        //VirtualKeyCode::Copyright => x11::keysym::XK_copyright,
        //VirtualKeyCode::Ordfeminine => x11::keysym::XK_ordfeminine,
        //VirtualKeyCode::Guillemotleft => x11::keysym::XK_guillemotleft,
        //VirtualKeyCode::Notsign => x11::keysym::XK_notsign,
        //VirtualKeyCode::Hyphen => x11::keysym::XK_hyphen,
        //VirtualKeyCode::Registered => x11::keysym::XK_registered,
        //VirtualKeyCode::Macron => x11::keysym::XK_macron,
        //VirtualKeyCode::Degree => x11::keysym::XK_degree,
        //VirtualKeyCode::Plusminus => x11::keysym::XK_plusminus,
        //VirtualKeyCode::Twosuperior => x11::keysym::XK_twosuperior,
        //VirtualKeyCode::Threesuperior => x11::keysym::XK_threesuperior,
        //VirtualKeyCode::Acute => x11::keysym::XK_acute,
        //VirtualKeyCode::Mu => x11::keysym::XK_mu,
        //VirtualKeyCode::Paragraph => x11::keysym::XK_paragraph,
        //VirtualKeyCode::Periodcentered => x11::keysym::XK_periodcentered,
        //VirtualKeyCode::Cedilla => x11::keysym::XK_cedilla,
        //VirtualKeyCode::Onesuperior => x11::keysym::XK_onesuperior,
        //VirtualKeyCode::Masculine => x11::keysym::XK_masculine,
        //VirtualKeyCode::Guillemotright => x11::keysym::XK_guillemotright,
        //VirtualKeyCode::Onequarter => x11::keysym::XK_onequarter,
        //VirtualKeyCode::Onehalf => x11::keysym::XK_onehalf,
        //VirtualKeyCode::Threequarters => x11::keysym::XK_threequarters,
        //VirtualKeyCode::Questiondown => x11::keysym::XK_questiondown,
        //VirtualKeyCode::Agrave => x11::keysym::XK_Agrave,
        //VirtualKeyCode::Aacute => x11::keysym::XK_Aacute,
        //VirtualKeyCode::Acircumflex => x11::keysym::XK_Acircumflex,
        //VirtualKeyCode::Atilde => x11::keysym::XK_Atilde,
        //VirtualKeyCode::Adiaeresis => x11::keysym::XK_Adiaeresis,
        //VirtualKeyCode::Aring => x11::keysym::XK_Aring,
        //VirtualKeyCode::Ae => x11::keysym::XK_AE,
        //VirtualKeyCode::Ccedilla => x11::keysym::XK_Ccedilla,
        //VirtualKeyCode::Egrave => x11::keysym::XK_Egrave,
        //VirtualKeyCode::Eacute => x11::keysym::XK_Eacute,
        //VirtualKeyCode::Ecircumflex => x11::keysym::XK_Ecircumflex,
        //VirtualKeyCode::Ediaeresis => x11::keysym::XK_Ediaeresis,
        //VirtualKeyCode::Igrave => x11::keysym::XK_Igrave,
        //VirtualKeyCode::Iacute => x11::keysym::XK_Iacute,
        //VirtualKeyCode::Icircumflex => x11::keysym::XK_Icircumflex,
        //VirtualKeyCode::Idiaeresis => x11::keysym::XK_Idiaeresis,
        //VirtualKeyCode::Eth => x11::keysym::XK_ETH,
        //VirtualKeyCode::Eth => x11::keysym::XK_Eth,
        //VirtualKeyCode::Ntilde => x11::keysym::XK_Ntilde,
        //VirtualKeyCode::Ograve => x11::keysym::XK_Ograve,
        //VirtualKeyCode::Oacute => x11::keysym::XK_Oacute,
        //VirtualKeyCode::Ocircumflex => x11::keysym::XK_Ocircumflex,
        //VirtualKeyCode::Otilde => x11::keysym::XK_Otilde,
        //VirtualKeyCode::Odiaeresis => x11::keysym::XK_Odiaeresis,
        //VirtualKeyCode::Multiply => x11::keysym::XK_multiply,
        //VirtualKeyCode::Ooblique => x11::keysym::XK_Ooblique,
        //VirtualKeyCode::Ugrave => x11::keysym::XK_Ugrave,
        //VirtualKeyCode::Uacute => x11::keysym::XK_Uacute,
        //VirtualKeyCode::Ucircumflex => x11::keysym::XK_Ucircumflex,
        //VirtualKeyCode::Udiaeresis => x11::keysym::XK_Udiaeresis,
        //VirtualKeyCode::Yacute => x11::keysym::XK_Yacute,
        //VirtualKeyCode::Thorn => x11::keysym::XK_THORN,
        //VirtualKeyCode::Thorn => x11::keysym::XK_Thorn,
        //VirtualKeyCode::Ssharp => x11::keysym::XK_ssharp,
        //VirtualKeyCode::Agrave => x11::keysym::XK_agrave,
        //VirtualKeyCode::Aacute => x11::keysym::XK_aacute,
        //VirtualKeyCode::Acircumflex => x11::keysym::XK_acircumflex,
        //VirtualKeyCode::Atilde => x11::keysym::XK_atilde,
        //VirtualKeyCode::Adiaeresis => x11::keysym::XK_adiaeresis,
        //VirtualKeyCode::Aring => x11::keysym::XK_aring,
        //VirtualKeyCode::Ae => x11::keysym::XK_ae,
        //VirtualKeyCode::Ccedilla => x11::keysym::XK_ccedilla,
        //VirtualKeyCode::Egrave => x11::keysym::XK_egrave,
        //VirtualKeyCode::Eacute => x11::keysym::XK_eacute,
        //VirtualKeyCode::Ecircumflex => x11::keysym::XK_ecircumflex,
        //VirtualKeyCode::Ediaeresis => x11::keysym::XK_ediaeresis,
        //VirtualKeyCode::Igrave => x11::keysym::XK_igrave,
        //VirtualKeyCode::Iacute => x11::keysym::XK_iacute,
        //VirtualKeyCode::Icircumflex => x11::keysym::XK_icircumflex,
        //VirtualKeyCode::Idiaeresis => x11::keysym::XK_idiaeresis,
        //VirtualKeyCode::Eth => x11::keysym::XK_eth,
        //VirtualKeyCode::Ntilde => x11::keysym::XK_ntilde,
        //VirtualKeyCode::Ograve => x11::keysym::XK_ograve,
        //VirtualKeyCode::Oacute => x11::keysym::XK_oacute,
        //VirtualKeyCode::Ocircumflex => x11::keysym::XK_ocircumflex,
        //VirtualKeyCode::Otilde => x11::keysym::XK_otilde,
        //VirtualKeyCode::Odiaeresis => x11::keysym::XK_odiaeresis,
        //VirtualKeyCode::Division => x11::keysym::XK_division,
        //VirtualKeyCode::Oslash => x11::keysym::XK_oslash,
        //VirtualKeyCode::Ugrave => x11::keysym::XK_ugrave,
        //VirtualKeyCode::Uacute => x11::keysym::XK_uacute,
        //VirtualKeyCode::Ucircumflex => x11::keysym::XK_ucircumflex,
        //VirtualKeyCode::Udiaeresis => x11::keysym::XK_udiaeresis,
        //VirtualKeyCode::Yacute => x11::keysym::XK_yacute,
        //VirtualKeyCode::Thorn => x11::keysym::XK_thorn,
        //VirtualKeyCode::Ydiaeresis => x11::keysym::XK_ydiaeresis,
        //VirtualKeyCode::Aogonek => x11::keysym::XK_Aogonek,
        //VirtualKeyCode::Breve => x11::keysym::XK_breve,
        //VirtualKeyCode::Lstroke => x11::keysym::XK_Lstroke,
        //VirtualKeyCode::Lcaron => x11::keysym::XK_Lcaron,
        //VirtualKeyCode::Sacute => x11::keysym::XK_Sacute,
        //VirtualKeyCode::Scaron => x11::keysym::XK_Scaron,
        //VirtualKeyCode::Scedilla => x11::keysym::XK_Scedilla,
        //VirtualKeyCode::Tcaron => x11::keysym::XK_Tcaron,
        //VirtualKeyCode::Zacute => x11::keysym::XK_Zacute,
        //VirtualKeyCode::Zcaron => x11::keysym::XK_Zcaron,
        //VirtualKeyCode::Zabovedot => x11::keysym::XK_Zabovedot,
        //VirtualKeyCode::Aogonek => x11::keysym::XK_aogonek,
        //VirtualKeyCode::Ogonek => x11::keysym::XK_ogonek,
        //VirtualKeyCode::Lstroke => x11::keysym::XK_lstroke,
        //VirtualKeyCode::Lcaron => x11::keysym::XK_lcaron,
        //VirtualKeyCode::Sacute => x11::keysym::XK_sacute,
        //VirtualKeyCode::Caron => x11::keysym::XK_caron,
        //VirtualKeyCode::Scaron => x11::keysym::XK_scaron,
        //VirtualKeyCode::Scedilla => x11::keysym::XK_scedilla,
        //VirtualKeyCode::Tcaron => x11::keysym::XK_tcaron,
        //VirtualKeyCode::Zacute => x11::keysym::XK_zacute,
        //VirtualKeyCode::Doubleacute => x11::keysym::XK_doubleacute,
        //VirtualKeyCode::Zcaron => x11::keysym::XK_zcaron,
        //VirtualKeyCode::Zabovedot => x11::keysym::XK_zabovedot,
        //VirtualKeyCode::Racute => x11::keysym::XK_Racute,
        //VirtualKeyCode::Abreve => x11::keysym::XK_Abreve,
        //VirtualKeyCode::Lacute => x11::keysym::XK_Lacute,
        //VirtualKeyCode::Cacute => x11::keysym::XK_Cacute,
        //VirtualKeyCode::Ccaron => x11::keysym::XK_Ccaron,
        //VirtualKeyCode::Eogonek => x11::keysym::XK_Eogonek,
        //VirtualKeyCode::Ecaron => x11::keysym::XK_Ecaron,
        //VirtualKeyCode::Dcaron => x11::keysym::XK_Dcaron,
        //VirtualKeyCode::Dstroke => x11::keysym::XK_Dstroke,
        //VirtualKeyCode::Nacute => x11::keysym::XK_Nacute,
        //VirtualKeyCode::Ncaron => x11::keysym::XK_Ncaron,
        //VirtualKeyCode::Odoubleacute => x11::keysym::XK_Odoubleacute,
        //VirtualKeyCode::Rcaron => x11::keysym::XK_Rcaron,
        //VirtualKeyCode::Uring => x11::keysym::XK_Uring,
        //VirtualKeyCode::Udoubleacute => x11::keysym::XK_Udoubleacute,
        //VirtualKeyCode::Tcedilla => x11::keysym::XK_Tcedilla,
        //VirtualKeyCode::Racute => x11::keysym::XK_racute,
        //VirtualKeyCode::Abreve => x11::keysym::XK_abreve,
        //VirtualKeyCode::Lacute => x11::keysym::XK_lacute,
        //VirtualKeyCode::Cacute => x11::keysym::XK_cacute,
        //VirtualKeyCode::Ccaron => x11::keysym::XK_ccaron,
        //VirtualKeyCode::Eogonek => x11::keysym::XK_eogonek,
        //VirtualKeyCode::Ecaron => x11::keysym::XK_ecaron,
        //VirtualKeyCode::Dcaron => x11::keysym::XK_dcaron,
        //VirtualKeyCode::Dstroke => x11::keysym::XK_dstroke,
        //VirtualKeyCode::Nacute => x11::keysym::XK_nacute,
        //VirtualKeyCode::Ncaron => x11::keysym::XK_ncaron,
        //VirtualKeyCode::Odoubleacute => x11::keysym::XK_odoubleacute,
        //VirtualKeyCode::Udoubleacute => x11::keysym::XK_udoubleacute,
        //VirtualKeyCode::Rcaron => x11::keysym::XK_rcaron,
        //VirtualKeyCode::Uring => x11::keysym::XK_uring,
        //VirtualKeyCode::Tcedilla => x11::keysym::XK_tcedilla,
        //VirtualKeyCode::Abovedot => x11::keysym::XK_abovedot,
        //VirtualKeyCode::Hstroke => x11::keysym::XK_Hstroke,
        //VirtualKeyCode::Hcircumflex => x11::keysym::XK_Hcircumflex,
        //VirtualKeyCode::Iabovedot => x11::keysym::XK_Iabovedot,
        //VirtualKeyCode::Gbreve => x11::keysym::XK_Gbreve,
        //VirtualKeyCode::Jcircumflex => x11::keysym::XK_Jcircumflex,
        //VirtualKeyCode::Hstroke => x11::keysym::XK_hstroke,
        //VirtualKeyCode::Hcircumflex => x11::keysym::XK_hcircumflex,
        //VirtualKeyCode::Idotless => x11::keysym::XK_idotless,
        //VirtualKeyCode::Gbreve => x11::keysym::XK_gbreve,
        //VirtualKeyCode::Jcircumflex => x11::keysym::XK_jcircumflex,
        //VirtualKeyCode::Cabovedot => x11::keysym::XK_Cabovedot,
        //VirtualKeyCode::Ccircumflex => x11::keysym::XK_Ccircumflex,
        //VirtualKeyCode::Gabovedot => x11::keysym::XK_Gabovedot,
        //VirtualKeyCode::Gcircumflex => x11::keysym::XK_Gcircumflex,
        //VirtualKeyCode::Ubreve => x11::keysym::XK_Ubreve,
        //VirtualKeyCode::Scircumflex => x11::keysym::XK_Scircumflex,
        //VirtualKeyCode::Cabovedot => x11::keysym::XK_cabovedot,
        //VirtualKeyCode::Ccircumflex => x11::keysym::XK_ccircumflex,
        //VirtualKeyCode::Gabovedot => x11::keysym::XK_gabovedot,
        //VirtualKeyCode::Gcircumflex => x11::keysym::XK_gcircumflex,
        //VirtualKeyCode::Ubreve => x11::keysym::XK_ubreve,
        //VirtualKeyCode::Scircumflex => x11::keysym::XK_scircumflex,
        //VirtualKeyCode::Kra => x11::keysym::XK_kra,
        //VirtualKeyCode::Kappa => x11::keysym::XK_kappa,
        //VirtualKeyCode::Rcedilla => x11::keysym::XK_Rcedilla,
        //VirtualKeyCode::Itilde => x11::keysym::XK_Itilde,
        //VirtualKeyCode::Lcedilla => x11::keysym::XK_Lcedilla,
        //VirtualKeyCode::Emacron => x11::keysym::XK_Emacron,
        //VirtualKeyCode::Gcedilla => x11::keysym::XK_Gcedilla,
        //VirtualKeyCode::Tslash => x11::keysym::XK_Tslash,
        //VirtualKeyCode::Rcedilla => x11::keysym::XK_rcedilla,
        //VirtualKeyCode::Itilde => x11::keysym::XK_itilde,
        //VirtualKeyCode::Lcedilla => x11::keysym::XK_lcedilla,
        //VirtualKeyCode::Emacron => x11::keysym::XK_emacron,
        //VirtualKeyCode::Gcedilla => x11::keysym::XK_gcedilla,
        //VirtualKeyCode::Tslash => x11::keysym::XK_tslash,
        //VirtualKeyCode::Eng => x11::keysym::XK_ENG,
        //VirtualKeyCode::Eng => x11::keysym::XK_eng,
        //VirtualKeyCode::Amacron => x11::keysym::XK_Amacron,
        //VirtualKeyCode::Iogonek => x11::keysym::XK_Iogonek,
        //VirtualKeyCode::Eabovedot => x11::keysym::XK_Eabovedot,
        //VirtualKeyCode::Imacron => x11::keysym::XK_Imacron,
        //VirtualKeyCode::Ncedilla => x11::keysym::XK_Ncedilla,
        //VirtualKeyCode::Omacron => x11::keysym::XK_Omacron,
        //VirtualKeyCode::Kcedilla => x11::keysym::XK_Kcedilla,
        //VirtualKeyCode::Uogonek => x11::keysym::XK_Uogonek,
        //VirtualKeyCode::Utilde => x11::keysym::XK_Utilde,
        //VirtualKeyCode::Umacron => x11::keysym::XK_Umacron,
        //VirtualKeyCode::Amacron => x11::keysym::XK_amacron,
        //VirtualKeyCode::Iogonek => x11::keysym::XK_iogonek,
        //VirtualKeyCode::Eabovedot => x11::keysym::XK_eabovedot,
        //VirtualKeyCode::Imacron => x11::keysym::XK_imacron,
        //VirtualKeyCode::Ncedilla => x11::keysym::XK_ncedilla,
        //VirtualKeyCode::Omacron => x11::keysym::XK_omacron,
        //VirtualKeyCode::Kcedilla => x11::keysym::XK_kcedilla,
        //VirtualKeyCode::Uogonek => x11::keysym::XK_uogonek,
        //VirtualKeyCode::Utilde => x11::keysym::XK_utilde,
        //VirtualKeyCode::Umacron => x11::keysym::XK_umacron,
        //VirtualKeyCode::Overline => x11::keysym::XK_overline,
        //VirtualKeyCode::Kana_fullstop => x11::keysym::XK_kana_fullstop,
        //VirtualKeyCode::Kana_openingbracket => x11::keysym::XK_kana_openingbracket,
        //VirtualKeyCode::Kana_closingbracket => x11::keysym::XK_kana_closingbracket,
        //VirtualKeyCode::Kana_comma => x11::keysym::XK_kana_comma,
        //VirtualKeyCode::Kana_conjunctive => x11::keysym::XK_kana_conjunctive,
        //VirtualKeyCode::Kana_middledot => x11::keysym::XK_kana_middledot,
        //VirtualKeyCode::Kana_wo => x11::keysym::XK_kana_WO,
        //VirtualKeyCode::Kana_a => x11::keysym::XK_kana_a,
        //VirtualKeyCode::Kana_i => x11::keysym::XK_kana_i,
        //VirtualKeyCode::Kana_u => x11::keysym::XK_kana_u,
        //VirtualKeyCode::Kana_e => x11::keysym::XK_kana_e,
        //VirtualKeyCode::Kana_o => x11::keysym::XK_kana_o,
        //VirtualKeyCode::Kana_ya => x11::keysym::XK_kana_ya,
        //VirtualKeyCode::Kana_yu => x11::keysym::XK_kana_yu,
        //VirtualKeyCode::Kana_yo => x11::keysym::XK_kana_yo,
        //VirtualKeyCode::Kana_tsu => x11::keysym::XK_kana_tsu,
        //VirtualKeyCode::Kana_tu => x11::keysym::XK_kana_tu,
        //VirtualKeyCode::Prolongedsound => x11::keysym::XK_prolongedsound,
        //VirtualKeyCode::Kana_a => x11::keysym::XK_kana_A,
        //VirtualKeyCode::Kana_i => x11::keysym::XK_kana_I,
        //VirtualKeyCode::Kana_u => x11::keysym::XK_kana_U,
        //VirtualKeyCode::Kana_e => x11::keysym::XK_kana_E,
        //VirtualKeyCode::Kana_o => x11::keysym::XK_kana_O,
        //VirtualKeyCode::Kana_ka => x11::keysym::XK_kana_KA,
        //VirtualKeyCode::Kana_ki => x11::keysym::XK_kana_KI,
        //VirtualKeyCode::Kana_ku => x11::keysym::XK_kana_KU,
        //VirtualKeyCode::Kana_ke => x11::keysym::XK_kana_KE,
        //VirtualKeyCode::Kana_ko => x11::keysym::XK_kana_KO,
        //VirtualKeyCode::Kana_sa => x11::keysym::XK_kana_SA,
        //VirtualKeyCode::Kana_shi => x11::keysym::XK_kana_SHI,
        //VirtualKeyCode::Kana_su => x11::keysym::XK_kana_SU,
        //VirtualKeyCode::Kana_se => x11::keysym::XK_kana_SE,
        //VirtualKeyCode::Kana_so => x11::keysym::XK_kana_SO,
        //VirtualKeyCode::Kana_ta => x11::keysym::XK_kana_TA,
        //VirtualKeyCode::Kana_chi => x11::keysym::XK_kana_CHI,
        //VirtualKeyCode::Kana_ti => x11::keysym::XK_kana_TI,
        //VirtualKeyCode::Kana_tsu => x11::keysym::XK_kana_TSU,
        //VirtualKeyCode::Kana_tu => x11::keysym::XK_kana_TU,
        //VirtualKeyCode::Kana_te => x11::keysym::XK_kana_TE,
        //VirtualKeyCode::Kana_to => x11::keysym::XK_kana_TO,
        //VirtualKeyCode::Kana_na => x11::keysym::XK_kana_NA,
        //VirtualKeyCode::Kana_ni => x11::keysym::XK_kana_NI,
        //VirtualKeyCode::Kana_nu => x11::keysym::XK_kana_NU,
        //VirtualKeyCode::Kana_ne => x11::keysym::XK_kana_NE,
        //VirtualKeyCode::Kana_no => x11::keysym::XK_kana_NO,
        //VirtualKeyCode::Kana_ha => x11::keysym::XK_kana_HA,
        //VirtualKeyCode::Kana_hi => x11::keysym::XK_kana_HI,
        //VirtualKeyCode::Kana_fu => x11::keysym::XK_kana_FU,
        //VirtualKeyCode::Kana_hu => x11::keysym::XK_kana_HU,
        //VirtualKeyCode::Kana_he => x11::keysym::XK_kana_HE,
        //VirtualKeyCode::Kana_ho => x11::keysym::XK_kana_HO,
        //VirtualKeyCode::Kana_ma => x11::keysym::XK_kana_MA,
        //VirtualKeyCode::Kana_mi => x11::keysym::XK_kana_MI,
        //VirtualKeyCode::Kana_mu => x11::keysym::XK_kana_MU,
        //VirtualKeyCode::Kana_me => x11::keysym::XK_kana_ME,
        //VirtualKeyCode::Kana_mo => x11::keysym::XK_kana_MO,
        //VirtualKeyCode::Kana_ya => x11::keysym::XK_kana_YA,
        //VirtualKeyCode::Kana_yu => x11::keysym::XK_kana_YU,
        //VirtualKeyCode::Kana_yo => x11::keysym::XK_kana_YO,
        //VirtualKeyCode::Kana_ra => x11::keysym::XK_kana_RA,
        //VirtualKeyCode::Kana_ri => x11::keysym::XK_kana_RI,
        //VirtualKeyCode::Kana_ru => x11::keysym::XK_kana_RU,
        //VirtualKeyCode::Kana_re => x11::keysym::XK_kana_RE,
        //VirtualKeyCode::Kana_ro => x11::keysym::XK_kana_RO,
        //VirtualKeyCode::Kana_wa => x11::keysym::XK_kana_WA,
        //VirtualKeyCode::Kana_n => x11::keysym::XK_kana_N,
        //VirtualKeyCode::Voicedsound => x11::keysym::XK_voicedsound,
        //VirtualKeyCode::Semivoicedsound => x11::keysym::XK_semivoicedsound,
        //VirtualKeyCode::Kana_switch => x11::keysym::XK_kana_switch,
        //VirtualKeyCode::Arabic_comma => x11::keysym::XK_Arabic_comma,
        //VirtualKeyCode::Arabic_semicolon => x11::keysym::XK_Arabic_semicolon,
        //VirtualKeyCode::Arabic_question_mark => x11::keysym::XK_Arabic_question_mark,
        //VirtualKeyCode::Arabic_hamza => x11::keysym::XK_Arabic_hamza,
        //VirtualKeyCode::Arabic_maddaonalef => x11::keysym::XK_Arabic_maddaonalef,
        //VirtualKeyCode::Arabic_hamzaonalef => x11::keysym::XK_Arabic_hamzaonalef,
        //VirtualKeyCode::Arabic_hamzaonwaw => x11::keysym::XK_Arabic_hamzaonwaw,
        //VirtualKeyCode::Arabic_hamzaunderalef => x11::keysym::XK_Arabic_hamzaunderalef,
        //VirtualKeyCode::Arabic_hamzaonyeh => x11::keysym::XK_Arabic_hamzaonyeh,
        //VirtualKeyCode::Arabic_alef => x11::keysym::XK_Arabic_alef,
        //VirtualKeyCode::Arabic_beh => x11::keysym::XK_Arabic_beh,
        //VirtualKeyCode::Arabic_tehmarbuta => x11::keysym::XK_Arabic_tehmarbuta,
        //VirtualKeyCode::Arabic_teh => x11::keysym::XK_Arabic_teh,
        //VirtualKeyCode::Arabic_theh => x11::keysym::XK_Arabic_theh,
        //VirtualKeyCode::Arabic_jeem => x11::keysym::XK_Arabic_jeem,
        //VirtualKeyCode::Arabic_hah => x11::keysym::XK_Arabic_hah,
        //VirtualKeyCode::Arabic_khah => x11::keysym::XK_Arabic_khah,
        //VirtualKeyCode::Arabic_dal => x11::keysym::XK_Arabic_dal,
        //VirtualKeyCode::Arabic_thal => x11::keysym::XK_Arabic_thal,
        //VirtualKeyCode::Arabic_ra => x11::keysym::XK_Arabic_ra,
        //VirtualKeyCode::Arabic_zain => x11::keysym::XK_Arabic_zain,
        //VirtualKeyCode::Arabic_seen => x11::keysym::XK_Arabic_seen,
        //VirtualKeyCode::Arabic_sheen => x11::keysym::XK_Arabic_sheen,
        //VirtualKeyCode::Arabic_sad => x11::keysym::XK_Arabic_sad,
        //VirtualKeyCode::Arabic_dad => x11::keysym::XK_Arabic_dad,
        //VirtualKeyCode::Arabic_tah => x11::keysym::XK_Arabic_tah,
        //VirtualKeyCode::Arabic_zah => x11::keysym::XK_Arabic_zah,
        //VirtualKeyCode::Arabic_ain => x11::keysym::XK_Arabic_ain,
        //VirtualKeyCode::Arabic_ghain => x11::keysym::XK_Arabic_ghain,
        //VirtualKeyCode::Arabic_tatweel => x11::keysym::XK_Arabic_tatweel,
        //VirtualKeyCode::Arabic_feh => x11::keysym::XK_Arabic_feh,
        //VirtualKeyCode::Arabic_qaf => x11::keysym::XK_Arabic_qaf,
        //VirtualKeyCode::Arabic_kaf => x11::keysym::XK_Arabic_kaf,
        //VirtualKeyCode::Arabic_lam => x11::keysym::XK_Arabic_lam,
        //VirtualKeyCode::Arabic_meem => x11::keysym::XK_Arabic_meem,
        //VirtualKeyCode::Arabic_noon => x11::keysym::XK_Arabic_noon,
        //VirtualKeyCode::Arabic_ha => x11::keysym::XK_Arabic_ha,
        //VirtualKeyCode::Arabic_heh => x11::keysym::XK_Arabic_heh,
        //VirtualKeyCode::Arabic_waw => x11::keysym::XK_Arabic_waw,
        //VirtualKeyCode::Arabic_alefmaksura => x11::keysym::XK_Arabic_alefmaksura,
        //VirtualKeyCode::Arabic_yeh => x11::keysym::XK_Arabic_yeh,
        //VirtualKeyCode::Arabic_fathatan => x11::keysym::XK_Arabic_fathatan,
        //VirtualKeyCode::Arabic_dammatan => x11::keysym::XK_Arabic_dammatan,
        //VirtualKeyCode::Arabic_kasratan => x11::keysym::XK_Arabic_kasratan,
        //VirtualKeyCode::Arabic_fatha => x11::keysym::XK_Arabic_fatha,
        //VirtualKeyCode::Arabic_damma => x11::keysym::XK_Arabic_damma,
        //VirtualKeyCode::Arabic_kasra => x11::keysym::XK_Arabic_kasra,
        //VirtualKeyCode::Arabic_shadda => x11::keysym::XK_Arabic_shadda,
        //VirtualKeyCode::Arabic_sukun => x11::keysym::XK_Arabic_sukun,
        //VirtualKeyCode::Arabic_switch => x11::keysym::XK_Arabic_switch,
        //VirtualKeyCode::Serbian_dje => x11::keysym::XK_Serbian_dje,
        //VirtualKeyCode::Macedonia_gje => x11::keysym::XK_Macedonia_gje,
        //VirtualKeyCode::Cyrillic_io => x11::keysym::XK_Cyrillic_io,
        //VirtualKeyCode::Ukrainian_ie => x11::keysym::XK_Ukrainian_ie,
        //VirtualKeyCode::Ukranian_je => x11::keysym::XK_Ukranian_je,
        //VirtualKeyCode::Macedonia_dse => x11::keysym::XK_Macedonia_dse,
        //VirtualKeyCode::Ukrainian_i => x11::keysym::XK_Ukrainian_i,
        //VirtualKeyCode::Ukranian_i => x11::keysym::XK_Ukranian_i,
        //VirtualKeyCode::Ukrainian_yi => x11::keysym::XK_Ukrainian_yi,
        //VirtualKeyCode::Ukranian_yi => x11::keysym::XK_Ukranian_yi,
        //VirtualKeyCode::Cyrillic_je => x11::keysym::XK_Cyrillic_je,
        //VirtualKeyCode::Serbian_je => x11::keysym::XK_Serbian_je,
        //VirtualKeyCode::Cyrillic_lje => x11::keysym::XK_Cyrillic_lje,
        //VirtualKeyCode::Serbian_lje => x11::keysym::XK_Serbian_lje,
        //VirtualKeyCode::Cyrillic_nje => x11::keysym::XK_Cyrillic_nje,
        //VirtualKeyCode::Serbian_nje => x11::keysym::XK_Serbian_nje,
        //VirtualKeyCode::Serbian_tshe => x11::keysym::XK_Serbian_tshe,
        //VirtualKeyCode::Macedonia_kje => x11::keysym::XK_Macedonia_kje,
        //VirtualKeyCode::Byelorussian_shortu => x11::keysym::XK_Byelorussian_shortu,
        //VirtualKeyCode::Cyrillic_dzhe => x11::keysym::XK_Cyrillic_dzhe,
        //VirtualKeyCode::Serbian_dze => x11::keysym::XK_Serbian_dze,
        //VirtualKeyCode::Numerosign => x11::keysym::XK_numerosign,
        //VirtualKeyCode::Serbian_dje => x11::keysym::XK_Serbian_DJE,
        //VirtualKeyCode::Macedonia_gje => x11::keysym::XK_Macedonia_GJE,
        //VirtualKeyCode::Cyrillic_io => x11::keysym::XK_Cyrillic_IO,
        //VirtualKeyCode::Ukrainian_ie => x11::keysym::XK_Ukrainian_IE,
        //VirtualKeyCode::Ukranian_je => x11::keysym::XK_Ukranian_JE,
        //VirtualKeyCode::Macedonia_dse => x11::keysym::XK_Macedonia_DSE,
        //VirtualKeyCode::Ukrainian_i => x11::keysym::XK_Ukrainian_I,
        //VirtualKeyCode::Ukranian_i => x11::keysym::XK_Ukranian_I,
        //VirtualKeyCode::Ukrainian_yi => x11::keysym::XK_Ukrainian_YI,
        //VirtualKeyCode::Ukranian_yi => x11::keysym::XK_Ukranian_YI,
        //VirtualKeyCode::Cyrillic_je => x11::keysym::XK_Cyrillic_JE,
        //VirtualKeyCode::Serbian_je => x11::keysym::XK_Serbian_JE,
        //VirtualKeyCode::Cyrillic_lje => x11::keysym::XK_Cyrillic_LJE,
        //VirtualKeyCode::Serbian_lje => x11::keysym::XK_Serbian_LJE,
        //VirtualKeyCode::Cyrillic_nje => x11::keysym::XK_Cyrillic_NJE,
        //VirtualKeyCode::Serbian_nje => x11::keysym::XK_Serbian_NJE,
        //VirtualKeyCode::Serbian_tshe => x11::keysym::XK_Serbian_TSHE,
        //VirtualKeyCode::Macedonia_kje => x11::keysym::XK_Macedonia_KJE,
        //VirtualKeyCode::Byelorussian_shortu => x11::keysym::XK_Byelorussian_SHORTU,
        //VirtualKeyCode::Cyrillic_dzhe => x11::keysym::XK_Cyrillic_DZHE,
        //VirtualKeyCode::Serbian_dze => x11::keysym::XK_Serbian_DZE,
        //VirtualKeyCode::Cyrillic_yu => x11::keysym::XK_Cyrillic_yu,
        //VirtualKeyCode::Cyrillic_a => x11::keysym::XK_Cyrillic_a,
        //VirtualKeyCode::Cyrillic_be => x11::keysym::XK_Cyrillic_be,
        //VirtualKeyCode::Cyrillic_tse => x11::keysym::XK_Cyrillic_tse,
        //VirtualKeyCode::Cyrillic_de => x11::keysym::XK_Cyrillic_de,
        //VirtualKeyCode::Cyrillic_ie => x11::keysym::XK_Cyrillic_ie,
        //VirtualKeyCode::Cyrillic_ef => x11::keysym::XK_Cyrillic_ef,
        //VirtualKeyCode::Cyrillic_ghe => x11::keysym::XK_Cyrillic_ghe,
        //VirtualKeyCode::Cyrillic_ha => x11::keysym::XK_Cyrillic_ha,
        //VirtualKeyCode::Cyrillic_i => x11::keysym::XK_Cyrillic_i,
        //VirtualKeyCode::Cyrillic_shorti => x11::keysym::XK_Cyrillic_shorti,
        //VirtualKeyCode::Cyrillic_ka => x11::keysym::XK_Cyrillic_ka,
        //VirtualKeyCode::Cyrillic_el => x11::keysym::XK_Cyrillic_el,
        //VirtualKeyCode::Cyrillic_em => x11::keysym::XK_Cyrillic_em,
        //VirtualKeyCode::Cyrillic_en => x11::keysym::XK_Cyrillic_en,
        //VirtualKeyCode::Cyrillic_o => x11::keysym::XK_Cyrillic_o,
        //VirtualKeyCode::Cyrillic_pe => x11::keysym::XK_Cyrillic_pe,
        //VirtualKeyCode::Cyrillic_ya => x11::keysym::XK_Cyrillic_ya,
        //VirtualKeyCode::Cyrillic_er => x11::keysym::XK_Cyrillic_er,
        //VirtualKeyCode::Cyrillic_es => x11::keysym::XK_Cyrillic_es,
        //VirtualKeyCode::Cyrillic_te => x11::keysym::XK_Cyrillic_te,
        //VirtualKeyCode::Cyrillic_u => x11::keysym::XK_Cyrillic_u,
        //VirtualKeyCode::Cyrillic_zhe => x11::keysym::XK_Cyrillic_zhe,
        //VirtualKeyCode::Cyrillic_ve => x11::keysym::XK_Cyrillic_ve,
        //VirtualKeyCode::Cyrillic_softsign => x11::keysym::XK_Cyrillic_softsign,
        //VirtualKeyCode::Cyrillic_yeru => x11::keysym::XK_Cyrillic_yeru,
        //VirtualKeyCode::Cyrillic_ze => x11::keysym::XK_Cyrillic_ze,
        //VirtualKeyCode::Cyrillic_sha => x11::keysym::XK_Cyrillic_sha,
        //VirtualKeyCode::Cyrillic_e => x11::keysym::XK_Cyrillic_e,
        //VirtualKeyCode::Cyrillic_shcha => x11::keysym::XK_Cyrillic_shcha,
        //VirtualKeyCode::Cyrillic_che => x11::keysym::XK_Cyrillic_che,
        //VirtualKeyCode::Cyrillic_hardsign => x11::keysym::XK_Cyrillic_hardsign,
        //VirtualKeyCode::Cyrillic_yu => x11::keysym::XK_Cyrillic_YU,
        //VirtualKeyCode::Cyrillic_a => x11::keysym::XK_Cyrillic_A,
        //VirtualKeyCode::Cyrillic_be => x11::keysym::XK_Cyrillic_BE,
        //VirtualKeyCode::Cyrillic_tse => x11::keysym::XK_Cyrillic_TSE,
        //VirtualKeyCode::Cyrillic_de => x11::keysym::XK_Cyrillic_DE,
        //VirtualKeyCode::Cyrillic_ie => x11::keysym::XK_Cyrillic_IE,
        //VirtualKeyCode::Cyrillic_ef => x11::keysym::XK_Cyrillic_EF,
        //VirtualKeyCode::Cyrillic_ghe => x11::keysym::XK_Cyrillic_GHE,
        //VirtualKeyCode::Cyrillic_ha => x11::keysym::XK_Cyrillic_HA,
        //VirtualKeyCode::Cyrillic_i => x11::keysym::XK_Cyrillic_I,
        //VirtualKeyCode::Cyrillic_shorti => x11::keysym::XK_Cyrillic_SHORTI,
        //VirtualKeyCode::Cyrillic_ka => x11::keysym::XK_Cyrillic_KA,
        //VirtualKeyCode::Cyrillic_el => x11::keysym::XK_Cyrillic_EL,
        //VirtualKeyCode::Cyrillic_em => x11::keysym::XK_Cyrillic_EM,
        //VirtualKeyCode::Cyrillic_en => x11::keysym::XK_Cyrillic_EN,
        //VirtualKeyCode::Cyrillic_o => x11::keysym::XK_Cyrillic_O,
        //VirtualKeyCode::Cyrillic_pe => x11::keysym::XK_Cyrillic_PE,
        //VirtualKeyCode::Cyrillic_ya => x11::keysym::XK_Cyrillic_YA,
        //VirtualKeyCode::Cyrillic_er => x11::keysym::XK_Cyrillic_ER,
        //VirtualKeyCode::Cyrillic_es => x11::keysym::XK_Cyrillic_ES,
        //VirtualKeyCode::Cyrillic_te => x11::keysym::XK_Cyrillic_TE,
        //VirtualKeyCode::Cyrillic_u => x11::keysym::XK_Cyrillic_U,
        //VirtualKeyCode::Cyrillic_zhe => x11::keysym::XK_Cyrillic_ZHE,
        //VirtualKeyCode::Cyrillic_ve => x11::keysym::XK_Cyrillic_VE,
        //VirtualKeyCode::Cyrillic_softsign => x11::keysym::XK_Cyrillic_SOFTSIGN,
        //VirtualKeyCode::Cyrillic_yeru => x11::keysym::XK_Cyrillic_YERU,
        //VirtualKeyCode::Cyrillic_ze => x11::keysym::XK_Cyrillic_ZE,
        //VirtualKeyCode::Cyrillic_sha => x11::keysym::XK_Cyrillic_SHA,
        //VirtualKeyCode::Cyrillic_e => x11::keysym::XK_Cyrillic_E,
        //VirtualKeyCode::Cyrillic_shcha => x11::keysym::XK_Cyrillic_SHCHA,
        //VirtualKeyCode::Cyrillic_che => x11::keysym::XK_Cyrillic_CHE,
        //VirtualKeyCode::Cyrillic_hardsign => x11::keysym::XK_Cyrillic_HARDSIGN,
        //VirtualKeyCode::Greek_alphaaccent => x11::keysym::XK_Greek_ALPHAaccent,
        //VirtualKeyCode::Greek_epsilonaccent => x11::keysym::XK_Greek_EPSILONaccent,
        //VirtualKeyCode::Greek_etaaccent => x11::keysym::XK_Greek_ETAaccent,
        //VirtualKeyCode::Greek_iotaaccent => x11::keysym::XK_Greek_IOTAaccent,
        //VirtualKeyCode::Greek_iotadiaeresis => x11::keysym::XK_Greek_IOTAdiaeresis,
        //VirtualKeyCode::Greek_omicronaccent => x11::keysym::XK_Greek_OMICRONaccent,
        //VirtualKeyCode::Greek_upsilonaccent => x11::keysym::XK_Greek_UPSILONaccent,
        //VirtualKeyCode::Greek_upsilondieresis => x11::keysym::XK_Greek_UPSILONdieresis,
        //VirtualKeyCode::Greek_omegaaccent => x11::keysym::XK_Greek_OMEGAaccent,
        //VirtualKeyCode::Greek_accentdieresis => x11::keysym::XK_Greek_accentdieresis,
        //VirtualKeyCode::Greek_horizbar => x11::keysym::XK_Greek_horizbar,
        //VirtualKeyCode::Greek_alphaaccent => x11::keysym::XK_Greek_alphaaccent,
        //VirtualKeyCode::Greek_epsilonaccent => x11::keysym::XK_Greek_epsilonaccent,
        //VirtualKeyCode::Greek_etaaccent => x11::keysym::XK_Greek_etaaccent,
        //VirtualKeyCode::Greek_iotaaccent => x11::keysym::XK_Greek_iotaaccent,
        //VirtualKeyCode::Greek_iotadieresis => x11::keysym::XK_Greek_iotadieresis,
        //VirtualKeyCode::Greek_iotaaccentdieresis => x11::keysym::XK_Greek_iotaaccentdieresis,
        //VirtualKeyCode::Greek_omicronaccent => x11::keysym::XK_Greek_omicronaccent,
        //VirtualKeyCode::Greek_upsilonaccent => x11::keysym::XK_Greek_upsilonaccent,
        //VirtualKeyCode::Greek_upsilondieresis => x11::keysym::XK_Greek_upsilondieresis,
        //VirtualKeyCode::Greek_upsilonaccentdieresis => x11::keysym::XK_Greek_upsilonaccentdieresis,
        //VirtualKeyCode::Greek_omegaaccent => x11::keysym::XK_Greek_omegaaccent,
        //VirtualKeyCode::Greek_alpha => x11::keysym::XK_Greek_ALPHA,
        //VirtualKeyCode::Greek_beta => x11::keysym::XK_Greek_BETA,
        //VirtualKeyCode::Greek_gamma => x11::keysym::XK_Greek_GAMMA,
        //VirtualKeyCode::Greek_delta => x11::keysym::XK_Greek_DELTA,
        //VirtualKeyCode::Greek_epsilon => x11::keysym::XK_Greek_EPSILON,
        //VirtualKeyCode::Greek_zeta => x11::keysym::XK_Greek_ZETA,
        //VirtualKeyCode::Greek_eta => x11::keysym::XK_Greek_ETA,
        //VirtualKeyCode::Greek_theta => x11::keysym::XK_Greek_THETA,
        //VirtualKeyCode::Greek_iota => x11::keysym::XK_Greek_IOTA,
        //VirtualKeyCode::Greek_kappa => x11::keysym::XK_Greek_KAPPA,
        //VirtualKeyCode::Greek_lamda => x11::keysym::XK_Greek_LAMDA,
        //VirtualKeyCode::Greek_lambda => x11::keysym::XK_Greek_LAMBDA,
        //VirtualKeyCode::Greek_mu => x11::keysym::XK_Greek_MU,
        //VirtualKeyCode::Greek_nu => x11::keysym::XK_Greek_NU,
        //VirtualKeyCode::Greek_xi => x11::keysym::XK_Greek_XI,
        //VirtualKeyCode::Greek_omicron => x11::keysym::XK_Greek_OMICRON,
        //VirtualKeyCode::Greek_pi => x11::keysym::XK_Greek_PI,
        //VirtualKeyCode::Greek_rho => x11::keysym::XK_Greek_RHO,
        //VirtualKeyCode::Greek_sigma => x11::keysym::XK_Greek_SIGMA,
        //VirtualKeyCode::Greek_tau => x11::keysym::XK_Greek_TAU,
        //VirtualKeyCode::Greek_upsilon => x11::keysym::XK_Greek_UPSILON,
        //VirtualKeyCode::Greek_phi => x11::keysym::XK_Greek_PHI,
        //VirtualKeyCode::Greek_chi => x11::keysym::XK_Greek_CHI,
        //VirtualKeyCode::Greek_psi => x11::keysym::XK_Greek_PSI,
        //VirtualKeyCode::Greek_omega => x11::keysym::XK_Greek_OMEGA,
        //VirtualKeyCode::Greek_alpha => x11::keysym::XK_Greek_alpha,
        //VirtualKeyCode::Greek_beta => x11::keysym::XK_Greek_beta,
        //VirtualKeyCode::Greek_gamma => x11::keysym::XK_Greek_gamma,
        //VirtualKeyCode::Greek_delta => x11::keysym::XK_Greek_delta,
        //VirtualKeyCode::Greek_epsilon => x11::keysym::XK_Greek_epsilon,
        //VirtualKeyCode::Greek_zeta => x11::keysym::XK_Greek_zeta,
        //VirtualKeyCode::Greek_eta => x11::keysym::XK_Greek_eta,
        //VirtualKeyCode::Greek_theta => x11::keysym::XK_Greek_theta,
        //VirtualKeyCode::Greek_iota => x11::keysym::XK_Greek_iota,
        //VirtualKeyCode::Greek_kappa => x11::keysym::XK_Greek_kappa,
        //VirtualKeyCode::Greek_lamda => x11::keysym::XK_Greek_lamda,
        //VirtualKeyCode::Greek_lambda => x11::keysym::XK_Greek_lambda,
        //VirtualKeyCode::Greek_mu => x11::keysym::XK_Greek_mu,
        //VirtualKeyCode::Greek_nu => x11::keysym::XK_Greek_nu,
        //VirtualKeyCode::Greek_xi => x11::keysym::XK_Greek_xi,
        //VirtualKeyCode::Greek_omicron => x11::keysym::XK_Greek_omicron,
        //VirtualKeyCode::Greek_pi => x11::keysym::XK_Greek_pi,
        //VirtualKeyCode::Greek_rho => x11::keysym::XK_Greek_rho,
        //VirtualKeyCode::Greek_sigma => x11::keysym::XK_Greek_sigma,
        //VirtualKeyCode::Greek_finalsmallsigma => x11::keysym::XK_Greek_finalsmallsigma,
        //VirtualKeyCode::Greek_tau => x11::keysym::XK_Greek_tau,
        //VirtualKeyCode::Greek_upsilon => x11::keysym::XK_Greek_upsilon,
        //VirtualKeyCode::Greek_phi => x11::keysym::XK_Greek_phi,
        //VirtualKeyCode::Greek_chi => x11::keysym::XK_Greek_chi,
        //VirtualKeyCode::Greek_psi => x11::keysym::XK_Greek_psi,
        //VirtualKeyCode::Greek_omega => x11::keysym::XK_Greek_omega,
        //VirtualKeyCode::Greek_switch => x11::keysym::XK_Greek_switch,
        //VirtualKeyCode::Leftradical => x11::keysym::XK_leftradical,
        //VirtualKeyCode::Topleftradical => x11::keysym::XK_topleftradical,
        //VirtualKeyCode::Horizconnector => x11::keysym::XK_horizconnector,
        //VirtualKeyCode::Topintegral => x11::keysym::XK_topintegral,
        //VirtualKeyCode::Botintegral => x11::keysym::XK_botintegral,
        //VirtualKeyCode::Vertconnector => x11::keysym::XK_vertconnector,
        //VirtualKeyCode::Topleftsqbracket => x11::keysym::XK_topleftsqbracket,
        //VirtualKeyCode::Botleftsqbracket => x11::keysym::XK_botleftsqbracket,
        //VirtualKeyCode::Toprightsqbracket => x11::keysym::XK_toprightsqbracket,
        //VirtualKeyCode::Botrightsqbracket => x11::keysym::XK_botrightsqbracket,
        //VirtualKeyCode::Topleftparens => x11::keysym::XK_topleftparens,
        //VirtualKeyCode::Botleftparens => x11::keysym::XK_botleftparens,
        //VirtualKeyCode::Toprightparens => x11::keysym::XK_toprightparens,
        //VirtualKeyCode::Botrightparens => x11::keysym::XK_botrightparens,
        //VirtualKeyCode::Leftmiddlecurlybrace => x11::keysym::XK_leftmiddlecurlybrace,
        //VirtualKeyCode::Rightmiddlecurlybrace => x11::keysym::XK_rightmiddlecurlybrace,
        //VirtualKeyCode::Topleftsummation => x11::keysym::XK_topleftsummation,
        //VirtualKeyCode::Botleftsummation => x11::keysym::XK_botleftsummation,
        //VirtualKeyCode::Topvertsummationconnector => x11::keysym::XK_topvertsummationconnector,
        //VirtualKeyCode::Botvertsummationconnector => x11::keysym::XK_botvertsummationconnector,
        //VirtualKeyCode::Toprightsummation => x11::keysym::XK_toprightsummation,
        //VirtualKeyCode::Botrightsummation => x11::keysym::XK_botrightsummation,
        //VirtualKeyCode::Rightmiddlesummation => x11::keysym::XK_rightmiddlesummation,
        //VirtualKeyCode::Lessthanequal => x11::keysym::XK_lessthanequal,
        //VirtualKeyCode::Notequal => x11::keysym::XK_notequal,
        //VirtualKeyCode::Greaterthanequal => x11::keysym::XK_greaterthanequal,
        //VirtualKeyCode::Integral => x11::keysym::XK_integral,
        //VirtualKeyCode::Therefore => x11::keysym::XK_therefore,
        //VirtualKeyCode::Variation => x11::keysym::XK_variation,
        //VirtualKeyCode::Infinity => x11::keysym::XK_infinity,
        //VirtualKeyCode::Nabla => x11::keysym::XK_nabla,
        //VirtualKeyCode::Approximate => x11::keysym::XK_approximate,
        //VirtualKeyCode::Similarequal => x11::keysym::XK_similarequal,
        //VirtualKeyCode::Ifonlyif => x11::keysym::XK_ifonlyif,
        //VirtualKeyCode::Implies => x11::keysym::XK_implies,
        //VirtualKeyCode::Identical => x11::keysym::XK_identical,
        //VirtualKeyCode::Radical => x11::keysym::XK_radical,
        //VirtualKeyCode::Includedin => x11::keysym::XK_includedin,
        //VirtualKeyCode::Includes => x11::keysym::XK_includes,
        //VirtualKeyCode::Intersection => x11::keysym::XK_intersection,
        //VirtualKeyCode::Union => x11::keysym::XK_union,
        //VirtualKeyCode::Logicaland => x11::keysym::XK_logicaland,
        //VirtualKeyCode::Logicalor => x11::keysym::XK_logicalor,
        //VirtualKeyCode::Partialderivative => x11::keysym::XK_partialderivative,
        //VirtualKeyCode::Function => x11::keysym::XK_function,
        //VirtualKeyCode::Leftarrow => x11::keysym::XK_leftarrow,
        //VirtualKeyCode::Uparrow => x11::keysym::XK_uparrow,
        //VirtualKeyCode::Rightarrow => x11::keysym::XK_rightarrow,
        //VirtualKeyCode::Downarrow => x11::keysym::XK_downarrow,
        //VirtualKeyCode::Blank => x11::keysym::XK_blank,
        //VirtualKeyCode::Soliddiamond => x11::keysym::XK_soliddiamond,
        //VirtualKeyCode::Checkerboard => x11::keysym::XK_checkerboard,
        //VirtualKeyCode::Ht => x11::keysym::XK_ht,
        //VirtualKeyCode::Ff => x11::keysym::XK_ff,
        //VirtualKeyCode::Cr => x11::keysym::XK_cr,
        //VirtualKeyCode::Lf => x11::keysym::XK_lf,
        //VirtualKeyCode::Nl => x11::keysym::XK_nl,
        //VirtualKeyCode::Vt => x11::keysym::XK_vt,
        //VirtualKeyCode::Lowrightcorner => x11::keysym::XK_lowrightcorner,
        //VirtualKeyCode::Uprightcorner => x11::keysym::XK_uprightcorner,
        //VirtualKeyCode::Upleftcorner => x11::keysym::XK_upleftcorner,
        //VirtualKeyCode::Lowleftcorner => x11::keysym::XK_lowleftcorner,
        //VirtualKeyCode::Crossinglines => x11::keysym::XK_crossinglines,
        //VirtualKeyCode::Horizlinescan1 => x11::keysym::XK_horizlinescan1,
        //VirtualKeyCode::Horizlinescan3 => x11::keysym::XK_horizlinescan3,
        //VirtualKeyCode::Horizlinescan5 => x11::keysym::XK_horizlinescan5,
        //VirtualKeyCode::Horizlinescan7 => x11::keysym::XK_horizlinescan7,
        //VirtualKeyCode::Horizlinescan9 => x11::keysym::XK_horizlinescan9,
        //VirtualKeyCode::Leftt => x11::keysym::XK_leftt,
        //VirtualKeyCode::Rightt => x11::keysym::XK_rightt,
        //VirtualKeyCode::Bott => x11::keysym::XK_bott,
        //VirtualKeyCode::Topt => x11::keysym::XK_topt,
        //VirtualKeyCode::Vertbar => x11::keysym::XK_vertbar,
        //VirtualKeyCode::Emspace => x11::keysym::XK_emspace,
        //VirtualKeyCode::Enspace => x11::keysym::XK_enspace,
        //VirtualKeyCode::Em3space => x11::keysym::XK_em3space,
        //VirtualKeyCode::Em4space => x11::keysym::XK_em4space,
        //VirtualKeyCode::Digitspace => x11::keysym::XK_digitspace,
        //VirtualKeyCode::Punctspace => x11::keysym::XK_punctspace,
        //VirtualKeyCode::Thinspace => x11::keysym::XK_thinspace,
        //VirtualKeyCode::Hairspace => x11::keysym::XK_hairspace,
        //VirtualKeyCode::Emdash => x11::keysym::XK_emdash,
        //VirtualKeyCode::Endash => x11::keysym::XK_endash,
        //VirtualKeyCode::Signifblank => x11::keysym::XK_signifblank,
        //VirtualKeyCode::Ellipsis => x11::keysym::XK_ellipsis,
        //VirtualKeyCode::Doubbaselinedot => x11::keysym::XK_doubbaselinedot,
        //VirtualKeyCode::Onethird => x11::keysym::XK_onethird,
        //VirtualKeyCode::Twothirds => x11::keysym::XK_twothirds,
        //VirtualKeyCode::Onefifth => x11::keysym::XK_onefifth,
        //VirtualKeyCode::Twofifths => x11::keysym::XK_twofifths,
        //VirtualKeyCode::Threefifths => x11::keysym::XK_threefifths,
        //VirtualKeyCode::Fourfifths => x11::keysym::XK_fourfifths,
        //VirtualKeyCode::Onesixth => x11::keysym::XK_onesixth,
        //VirtualKeyCode::Fivesixths => x11::keysym::XK_fivesixths,
        //VirtualKeyCode::Careof => x11::keysym::XK_careof,
        //VirtualKeyCode::Figdash => x11::keysym::XK_figdash,
        //VirtualKeyCode::Leftanglebracket => x11::keysym::XK_leftanglebracket,
        //VirtualKeyCode::Decimalpoint => x11::keysym::XK_decimalpoint,
        //VirtualKeyCode::Rightanglebracket => x11::keysym::XK_rightanglebracket,
        //VirtualKeyCode::Marker => x11::keysym::XK_marker,
        //VirtualKeyCode::Oneeighth => x11::keysym::XK_oneeighth,
        //VirtualKeyCode::Threeeighths => x11::keysym::XK_threeeighths,
        //VirtualKeyCode::Fiveeighths => x11::keysym::XK_fiveeighths,
        //VirtualKeyCode::Seveneighths => x11::keysym::XK_seveneighths,
        //VirtualKeyCode::Trademark => x11::keysym::XK_trademark,
        //VirtualKeyCode::Signaturemark => x11::keysym::XK_signaturemark,
        //VirtualKeyCode::Trademarkincircle => x11::keysym::XK_trademarkincircle,
        //VirtualKeyCode::Leftopentriangle => x11::keysym::XK_leftopentriangle,
        //VirtualKeyCode::Rightopentriangle => x11::keysym::XK_rightopentriangle,
        //VirtualKeyCode::Emopencircle => x11::keysym::XK_emopencircle,
        //VirtualKeyCode::Emopenrectangle => x11::keysym::XK_emopenrectangle,
        //VirtualKeyCode::Leftsinglequotemark => x11::keysym::XK_leftsinglequotemark,
        //VirtualKeyCode::Rightsinglequotemark => x11::keysym::XK_rightsinglequotemark,
        //VirtualKeyCode::Leftdoublequotemark => x11::keysym::XK_leftdoublequotemark,
        //VirtualKeyCode::Rightdoublequotemark => x11::keysym::XK_rightdoublequotemark,
        //VirtualKeyCode::Prescription => x11::keysym::XK_prescription,
        //VirtualKeyCode::Minutes => x11::keysym::XK_minutes,
        //VirtualKeyCode::Seconds => x11::keysym::XK_seconds,
        //VirtualKeyCode::Latincross => x11::keysym::XK_latincross,
        //VirtualKeyCode::Hexagram => x11::keysym::XK_hexagram,
        //VirtualKeyCode::Filledrectbullet => x11::keysym::XK_filledrectbullet,
        //VirtualKeyCode::Filledlefttribullet => x11::keysym::XK_filledlefttribullet,
        //VirtualKeyCode::Filledrighttribullet => x11::keysym::XK_filledrighttribullet,
        //VirtualKeyCode::Emfilledcircle => x11::keysym::XK_emfilledcircle,
        //VirtualKeyCode::Emfilledrect => x11::keysym::XK_emfilledrect,
        //VirtualKeyCode::Enopencircbullet => x11::keysym::XK_enopencircbullet,
        //VirtualKeyCode::Enopensquarebullet => x11::keysym::XK_enopensquarebullet,
        //VirtualKeyCode::Openrectbullet => x11::keysym::XK_openrectbullet,
        //VirtualKeyCode::Opentribulletup => x11::keysym::XK_opentribulletup,
        //VirtualKeyCode::Opentribulletdown => x11::keysym::XK_opentribulletdown,
        //VirtualKeyCode::Openstar => x11::keysym::XK_openstar,
        //VirtualKeyCode::Enfilledcircbullet => x11::keysym::XK_enfilledcircbullet,
        //VirtualKeyCode::Enfilledsqbullet => x11::keysym::XK_enfilledsqbullet,
        //VirtualKeyCode::Filledtribulletup => x11::keysym::XK_filledtribulletup,
        //VirtualKeyCode::Filledtribulletdown => x11::keysym::XK_filledtribulletdown,
        //VirtualKeyCode::Leftpointer => x11::keysym::XK_leftpointer,
        //VirtualKeyCode::Rightpointer => x11::keysym::XK_rightpointer,
        //VirtualKeyCode::Club => x11::keysym::XK_club,
        //VirtualKeyCode::Diamond => x11::keysym::XK_diamond,
        //VirtualKeyCode::Heart => x11::keysym::XK_heart,
        //VirtualKeyCode::Maltesecross => x11::keysym::XK_maltesecross,
        //VirtualKeyCode::Dagger => x11::keysym::XK_dagger,
        //VirtualKeyCode::Doubledagger => x11::keysym::XK_doubledagger,
        //VirtualKeyCode::Checkmark => x11::keysym::XK_checkmark,
        //VirtualKeyCode::Ballotcross => x11::keysym::XK_ballotcross,
        //VirtualKeyCode::Musicalsharp => x11::keysym::XK_musicalsharp,
        //VirtualKeyCode::Musicalflat => x11::keysym::XK_musicalflat,
        //VirtualKeyCode::Malesymbol => x11::keysym::XK_malesymbol,
        //VirtualKeyCode::Femalesymbol => x11::keysym::XK_femalesymbol,
        //VirtualKeyCode::Telephone => x11::keysym::XK_telephone,
        //VirtualKeyCode::Telephonerecorder => x11::keysym::XK_telephonerecorder,
        //VirtualKeyCode::Phonographcopyright => x11::keysym::XK_phonographcopyright,
        //VirtualKeyCode::Caret => x11::keysym::XK_caret,
        //VirtualKeyCode::Singlelowquotemark => x11::keysym::XK_singlelowquotemark,
        //VirtualKeyCode::Doublelowquotemark => x11::keysym::XK_doublelowquotemark,
        //VirtualKeyCode::Cursor => x11::keysym::XK_cursor,
        //VirtualKeyCode::Leftcaret => x11::keysym::XK_leftcaret,
        //VirtualKeyCode::Rightcaret => x11::keysym::XK_rightcaret,
        //VirtualKeyCode::Downcaret => x11::keysym::XK_downcaret,
        //VirtualKeyCode::Upcaret => x11::keysym::XK_upcaret,
        //VirtualKeyCode::Overbar => x11::keysym::XK_overbar,
        //VirtualKeyCode::Downtack => x11::keysym::XK_downtack,
        //VirtualKeyCode::Upshoe => x11::keysym::XK_upshoe,
        //VirtualKeyCode::Downstile => x11::keysym::XK_downstile,
        //VirtualKeyCode::Underbar => x11::keysym::XK_underbar,
        //VirtualKeyCode::Jot => x11::keysym::XK_jot,
        //VirtualKeyCode::Quad => x11::keysym::XK_quad,
        //VirtualKeyCode::Uptack => x11::keysym::XK_uptack,
        //VirtualKeyCode::Circle => x11::keysym::XK_circle,
        //VirtualKeyCode::Upstile => x11::keysym::XK_upstile,
        //VirtualKeyCode::Downshoe => x11::keysym::XK_downshoe,
        //VirtualKeyCode::Rightshoe => x11::keysym::XK_rightshoe,
        //VirtualKeyCode::Leftshoe => x11::keysym::XK_leftshoe,
        //VirtualKeyCode::Lefttack => x11::keysym::XK_lefttack,
        //VirtualKeyCode::Righttack => x11::keysym::XK_righttack,
        //VirtualKeyCode::Hebrew_doublelowline => x11::keysym::XK_hebrew_doublelowline,
        //VirtualKeyCode::Hebrew_aleph => x11::keysym::XK_hebrew_aleph,
        //VirtualKeyCode::Hebrew_bet => x11::keysym::XK_hebrew_bet,
        //VirtualKeyCode::Hebrew_beth => x11::keysym::XK_hebrew_beth,
        //VirtualKeyCode::Hebrew_gimel => x11::keysym::XK_hebrew_gimel,
        //VirtualKeyCode::Hebrew_gimmel => x11::keysym::XK_hebrew_gimmel,
        //VirtualKeyCode::Hebrew_dalet => x11::keysym::XK_hebrew_dalet,
        //VirtualKeyCode::Hebrew_daleth => x11::keysym::XK_hebrew_daleth,
        //VirtualKeyCode::Hebrew_he => x11::keysym::XK_hebrew_he,
        //VirtualKeyCode::Hebrew_waw => x11::keysym::XK_hebrew_waw,
        //VirtualKeyCode::Hebrew_zain => x11::keysym::XK_hebrew_zain,
        //VirtualKeyCode::Hebrew_zayin => x11::keysym::XK_hebrew_zayin,
        //VirtualKeyCode::Hebrew_chet => x11::keysym::XK_hebrew_chet,
        //VirtualKeyCode::Hebrew_het => x11::keysym::XK_hebrew_het,
        //VirtualKeyCode::Hebrew_tet => x11::keysym::XK_hebrew_tet,
        //VirtualKeyCode::Hebrew_teth => x11::keysym::XK_hebrew_teth,
        //VirtualKeyCode::Hebrew_yod => x11::keysym::XK_hebrew_yod,
        //VirtualKeyCode::Hebrew_finalkaph => x11::keysym::XK_hebrew_finalkaph,
        //VirtualKeyCode::Hebrew_kaph => x11::keysym::XK_hebrew_kaph,
        //VirtualKeyCode::Hebrew_lamed => x11::keysym::XK_hebrew_lamed,
        //VirtualKeyCode::Hebrew_finalmem => x11::keysym::XK_hebrew_finalmem,
        //VirtualKeyCode::Hebrew_mem => x11::keysym::XK_hebrew_mem,
        //VirtualKeyCode::Hebrew_finalnun => x11::keysym::XK_hebrew_finalnun,
        //VirtualKeyCode::Hebrew_nun => x11::keysym::XK_hebrew_nun,
        //VirtualKeyCode::Hebrew_samech => x11::keysym::XK_hebrew_samech,
        //VirtualKeyCode::Hebrew_samekh => x11::keysym::XK_hebrew_samekh,
        //VirtualKeyCode::Hebrew_ayin => x11::keysym::XK_hebrew_ayin,
        //VirtualKeyCode::Hebrew_finalpe => x11::keysym::XK_hebrew_finalpe,
        //VirtualKeyCode::Hebrew_pe => x11::keysym::XK_hebrew_pe,
        //VirtualKeyCode::Hebrew_finalzade => x11::keysym::XK_hebrew_finalzade,
        //VirtualKeyCode::Hebrew_finalzadi => x11::keysym::XK_hebrew_finalzadi,
        //VirtualKeyCode::Hebrew_zade => x11::keysym::XK_hebrew_zade,
        //VirtualKeyCode::Hebrew_zadi => x11::keysym::XK_hebrew_zadi,
        //VirtualKeyCode::Hebrew_qoph => x11::keysym::XK_hebrew_qoph,
        //VirtualKeyCode::Hebrew_kuf => x11::keysym::XK_hebrew_kuf,
        //VirtualKeyCode::Hebrew_resh => x11::keysym::XK_hebrew_resh,
        //VirtualKeyCode::Hebrew_shin => x11::keysym::XK_hebrew_shin,
        //VirtualKeyCode::Hebrew_taw => x11::keysym::XK_hebrew_taw,
        //VirtualKeyCode::Hebrew_taf => x11::keysym::XK_hebrew_taf,
        //VirtualKeyCode::Hebrew_switch => x11::keysym::XK_Hebrew_switch,
        VirtualKeyCode::NavigateBackward => x11::keysym::XF86XK_Back,
        VirtualKeyCode::NavigateForward => x11::keysym::XF86XK_Forward,
        VirtualKeyCode::Copy => x11::keysym::XF86XK_Copy,
        VirtualKeyCode::Paste => x11::keysym::XF86XK_Paste,
        VirtualKeyCode::Cut => x11::keysym::XF86XK_Cut,
        _ => return None,
    })
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_keysym_to_vkc() {
        let keysym: XKeySym = VirtualKeyCode::W.into();
        let keycode: VirtualKeyCode = keysym.into();

        assert_eq!(keycode, VirtualKeyCode::W);

        let keysym2: XKeySym = keycode.into();

        assert_eq!(keysym2, keysym);
        assert_eq!(&x11::keysym::XK_W, keysym.as_ref());
    }
}
